"""
This type stub file was generated by pyright.
"""

import abc
from decorator import decorator
from traitlets.config.configurable import Configurable
from ..lib import pretty
from traitlets import Integer
from typing import Any

"""Display formatters.

Inheritance diagram:

.. inheritance-diagram:: IPython.core.formatters
   :parts: 3
"""
class DisplayFormatter(Configurable):
    active_types = ...
    ipython_display_formatter = ...
    mimebundle_formatter = ...
    formatters = ...
    def format(self, obj, include=..., exclude=...):
        """Return a format data dict for an object.

        By default all format types will be computed.

        The following MIME types are usually implemented:

        * text/plain
        * text/html
        * text/markdown
        * text/latex
        * application/json
        * application/javascript
        * application/pdf
        * image/png
        * image/jpeg
        * image/svg+xml

        Parameters
        ----------
        obj : object
            The Python object whose format data will be computed.
        include : list, tuple or set; optional
            A list of format type strings (MIME types) to include in the
            format data dict. If this is set *only* the format types included
            in this list will be computed.
        exclude : list, tuple or set; optional
            A list of format type string (MIME types) to exclude in the format
            data dict. If this is set all format types will be computed,
            except for those included in this argument.
            Mimetypes present in exclude will take precedence over the ones in include

        Returns
        -------
        (format_dict, metadata_dict) : tuple of two dicts
            format_dict is a dictionary of key/value pairs, one of each format that was
            generated for the object. The keys are the format types, which
            will usually be MIME type strings and the values and JSON'able
            data structure containing the raw data for the representation in
            that format.

            metadata_dict is a dictionary of metadata about each mime-type output.
            Its keys will be a strict subset of the keys in format_dict.

        Notes
        -----
            If an object implement `_repr_mimebundle_` as well as various
            `_repr_*_`, the data returned by `_repr_mimebundle_` will take
            precedence and the corresponding `_repr_*_` for this mimetype will
            not be called.

        """
        ...
    
    @property
    def format_types(self):
        """Return the format types (MIME types) of the active formatters."""
        ...
    


class FormatterWarning(UserWarning):
    """Warning class for errors in formatters"""
    ...


@decorator
def catch_format_error(method, self, *args, **kwargs):
    """show traceback on failed format call"""
    ...

class FormatterABC(metaclass=abc.ABCMeta):
    """ Abstract base class for Formatters.

    A formatter is a callable class that is responsible for computing the
    raw format data for a particular format type (MIME type). For example,
    an HTML formatter would have a format type of `text/html` and would return
    the HTML representation of the object when called.
    """
    format_type = ...
    enabled = ...
    @abc.abstractmethod
    def __call__(self, obj):
        """Return a JSON'able representation of the object.

        If the object cannot be formatted by this formatter,
        warn and return None.
        """
        ...
    


_raise_key_error = ...
class BaseFormatter(Configurable):
    """A base formatter class that is configurable.

    This formatter should usually be used as the base class of all formatters.
    It is a traited :class:`Configurable` class and includes an extensible
    API for users to determine how their objects are formatted. The following
    logic is used to find a function to format an given object.

    1. The object is introspected to see if it has a method with the name
       :attr:`print_method`. If is does, that object is passed to that method
       for formatting.
    2. If no print method is found, three internal dictionaries are consulted
       to find print method: :attr:`singleton_printers`, :attr:`type_printers`
       and :attr:`deferred_printers`.

    Users should use these dictionaries to register functions that will be
    used to compute the format data for their objects (if those objects don't
    have the special print methods). The easiest way of using these
    dictionaries is through the :meth:`for_type` and :meth:`for_type_by_name`
    methods.

    If no function/callable is found to compute the format data, ``None`` is
    returned and this format type is not used.
    """
    format_type = ...
    _return_type: Any = ...
    enabled = ...
    print_method = ...
    singleton_printers = ...
    type_printers = ...
    deferred_printers = ...
    @catch_format_error
    def __call__(self, obj): # -> None:
        """Compute the format for an object."""
        ...
    
    def __contains__(self, typ):
        """map in to lookup_by_type"""
        ...
    
    def lookup(self, obj):
        """Look up the formatter for a given instance.

        Parameters
        ----------
        obj : object instance

        Returns
        -------
        f : callable
            The registered formatting callable for the type.

        Raises
        ------
        KeyError if the type has not been registered.
        """
        ...
    
    def lookup_by_type(self, typ):
        """Look up the registered formatter for a type.

        Parameters
        ----------
        typ : type or '__module__.__name__' string for a type

        Returns
        -------
        f : callable
            The registered formatting callable for the type.

        Raises
        ------
        KeyError if the type has not been registered.
        """
        ...
    
    def for_type(self, typ, func=...): # -> None:
        """Add a format function for a given type.

        Parameters
        ----------
        typ : type or '__module__.__name__' string for a type
            The class of the object that will be formatted using `func`.

        func : callable
            A callable for computing the format data.
            `func` will be called with the object to be formatted,
            and will return the raw data in this formatter's format.
            Subclasses may use a different call signature for the
            `func` argument.

            If `func` is None or not specified, there will be no change,
            only returning the current value.

        Returns
        -------
        oldfunc : callable
            The currently registered callable.
            If you are registering a new formatter,
            this will be the previous value (to enable restoring later).
        """
        ...
    
    def for_type_by_name(self, type_module, type_name, func=...): # -> None:
        """Add a format function for a type specified by the full dotted
        module and name of the type, rather than the type of the object.

        Parameters
        ----------
        type_module : str
            The full dotted name of the module the type is defined in, like
            ``numpy``.

        type_name : str
            The name of the type (the class name), like ``dtype``

        func : callable
            A callable for computing the format data.
            `func` will be called with the object to be formatted,
            and will return the raw data in this formatter's format.
            Subclasses may use a different call signature for the
            `func` argument.

            If `func` is None or unspecified, there will be no change,
            only returning the current value.

        Returns
        -------
        oldfunc : callable
            The currently registered callable.
            If you are registering a new formatter,
            this will be the previous value (to enable restoring later).
        """
        ...
    
    def pop(self, typ, default=...): # -> Sentinel:
        """Pop a formatter for the given type.

        Parameters
        ----------
        typ : type or '__module__.__name__' string for a type
        default : object
            value to be returned if no formatter is registered for typ.

        Returns
        -------
        obj : object
            The last registered object for the type.

        Raises
        ------
        KeyError if the type is not registered and default is not specified.
        """
        ...
    


class PlainTextFormatter(BaseFormatter):
    """The default pretty-printer.

    This uses :mod:`IPython.lib.pretty` to compute the format data of
    the object. If the object cannot be pretty printed, :func:`repr` is used.
    See the documentation of :mod:`IPython.lib.pretty` for details on
    how to write pretty printers.  Here is a simple example::

        def dtype_pprinter(obj, p, cycle):
            if cycle:
                return p.text('dtype(...)')
            if hasattr(obj, 'fields'):
                if obj.fields is None:
                    p.text(repr(obj))
                else:
                    p.begin_group(7, 'dtype([')
                    for i, field in enumerate(obj.descr):
                        if i > 0:
                            p.text(',')
                            p.breakable()
                        p.pretty(field)
                    p.end_group(7, '])')
    """
    format_type = ...
    enabled = ...
    max_seq_length = Integer(pretty.MAX_SEQ_LENGTH, help="""Truncate large collections (lists, dicts, tuples, sets) to this size.
        
        Set to 0 to disable truncation.
        """).tag(config=True)
    print_method = ...
    pprint = ...
    verbose = ...
    max_width = Integer(79).tag(config=True)
    newline = ...
    float_format = ...
    float_precision = ...
    @catch_format_error
    def __call__(self, obj):
        """Compute the pretty representation of the object."""
        ...
    


class HTMLFormatter(BaseFormatter):
    """An HTML formatter.

    To define the callables that compute the HTML representation of your
    objects, define a :meth:`_repr_html_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be a valid HTML snippet that
    could be injected into an existing DOM. It should *not* include the
    ```<html>`` or ```<body>`` tags.
    """
    format_type = ...
    print_method = ...


class MarkdownFormatter(BaseFormatter):
    """A Markdown formatter.

    To define the callables that compute the Markdown representation of your
    objects, define a :meth:`_repr_markdown_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be a valid Markdown.
    """
    format_type = ...
    print_method = ...


class SVGFormatter(BaseFormatter):
    """An SVG formatter.

    To define the callables that compute the SVG representation of your
    objects, define a :meth:`_repr_svg_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be valid SVG enclosed in
    ```<svg>``` tags, that could be injected into an existing DOM. It should
    *not* include the ```<html>`` or ```<body>`` tags.
    """
    format_type = ...
    print_method = ...


class PNGFormatter(BaseFormatter):
    """A PNG formatter.

    To define the callables that compute the PNG representation of your
    objects, define a :meth:`_repr_png_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be raw PNG data, *not*
    base64 encoded.
    """
    format_type = ...
    print_method = ...
    _return_type = ...


class JPEGFormatter(BaseFormatter):
    """A JPEG formatter.

    To define the callables that compute the JPEG representation of your
    objects, define a :meth:`_repr_jpeg_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be raw JPEG data, *not*
    base64 encoded.
    """
    format_type = ...
    print_method = ...
    _return_type = ...


class LatexFormatter(BaseFormatter):
    """A LaTeX formatter.

    To define the callables that compute the LaTeX representation of your
    objects, define a :meth:`_repr_latex_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be a valid LaTeX equation,
    enclosed in either ```$```, ```$$``` or another LaTeX equation
    environment.
    """
    format_type = ...
    print_method = ...


class JSONFormatter(BaseFormatter):
    """A JSON string formatter.

    To define the callables that compute the JSONable representation of
    your objects, define a :meth:`_repr_json_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be a JSONable list or dict.
    JSON scalars (None, number, string) are not allowed, only dict or list containers.
    """
    format_type = ...
    _return_type = ...
    print_method = ...


class JavascriptFormatter(BaseFormatter):
    """A Javascript formatter.

    To define the callables that compute the Javascript representation of
    your objects, define a :meth:`_repr_javascript_` method or use the
    :meth:`for_type` or :meth:`for_type_by_name` methods to register functions
    that handle this.

    The return value of this formatter should be valid Javascript code and
    should *not* be enclosed in ```<script>``` tags.
    """
    format_type = ...
    print_method = ...


class PDFFormatter(BaseFormatter):
    """A PDF formatter.

    To define the callables that compute the PDF representation of your
    objects, define a :meth:`_repr_pdf_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    The return value of this formatter should be raw PDF data, *not*
    base64 encoded.
    """
    format_type = ...
    print_method = ...
    _return_type = ...


class IPythonDisplayFormatter(BaseFormatter):
    """An escape-hatch Formatter for objects that know how to display themselves.
    
    To define the callables that compute the representation of your
    objects, define a :meth:`_ipython_display_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this. Unlike mime-type displays, this method should not return anything,
    instead calling any appropriate display methods itself.
    
    This display formatter has highest priority.
    If it fires, no other display formatter will be called.

    Prior to IPython 6.1, `_ipython_display_` was the only way to display custom mime-types
    without registering a new Formatter.
    
    IPython 6.1 introduces `_repr_mimebundle_` for displaying custom mime-types,
    so `_ipython_display_` should only be used for objects that require unusual
    display patterns, such as multiple display calls.
    """
    print_method = ...
    _return_type = ...
    @catch_format_error
    def __call__(self, obj): # -> None:
        """Compute the format for an object."""
        ...
    


class MimeBundleFormatter(BaseFormatter):
    """A Formatter for arbitrary mime-types.

    Unlike other `_repr_<mimetype>_` methods,
    `_repr_mimebundle_` should return mime-bundle data,
    either the mime-keyed `data` dictionary or the tuple `(data, metadata)`.
    Any mime-type is valid.

    To define the callables that compute the mime-bundle representation of your
    objects, define a :meth:`_repr_mimebundle_` method or use the :meth:`for_type`
    or :meth:`for_type_by_name` methods to register functions that handle
    this.

    .. versionadded:: 6.1
    """
    print_method = ...
    _return_type = ...
    @catch_format_error
    def __call__(self, obj, include=..., exclude=...): # -> None:
        """Compute the format for an object.

        Identical to parent's method but we pass extra parameters to the method.

        Unlike other _repr_*_ `_repr_mimebundle_` should allow extra kwargs, in
        particular `include` and `exclude`.
        """
        ...
    


def format_display_data(obj, include=..., exclude=...):
    """Return a format data dict for an object.

    By default all format types will be computed.

    Parameters
    ----------
    obj : object
        The Python object whose format data will be computed.

    Returns
    -------
    format_dict : dict
        A dictionary of key/value pairs, one or each format that was
        generated for the object. The keys are the format types, which
        will usually be MIME type strings and the values and JSON'able
        data structure containing the raw data for the representation in
        that format.
    include : list or tuple, optional
        A list of format type strings (MIME types) to include in the
        format data dict. If this is set *only* the format types included
        in this list will be computed.
    exclude : list or tuple, optional
        A list of format type string (MIME types) to exclude in the format
        data dict. If this is set all format types will be computed,
        except for those included in this argument.
    """
    ...




import numpy as np
from skimage._shared.utils import deprecate_kwarg

from collections import deque

_param_options = ...

@deprecate_kwarg({"array": "image"}, deprecated_version="0.18", removed_version="0.20")
def find_contours(
    image,
    level: float | None = None,
    fully_connected="low",
    positive_orientation="low",
    *,
    mask=None
): ...
def _assemble_contours(segments): ...




from numpy.typing import ArrayLike
from ..visuals.transforms.chain import ChainTransform
from ..visuals.transforms.transform_system import TransformSystem

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import weakref

from ..util.event import Event, EmitterGroup
from ..visuals.transforms import (
    NullTransform,
    BaseTransform,
    ChainTransform,
    create_transform,
    TransformSystem,
)

class Node(object):

    # Needed to allow subclasses to repr() themselves before Node.__init__()
    _name: None = ...

    def __init__(
        self,
        parent: Node | None = None,
        name: str | None = None,
        transforms: None | TransformSystem = None,
    ): ...
    @property
    def visible(self): ...
    @visible.setter
    def visible(self, val): ...
    @property
    def name(self): ...
    @name.setter
    def name(self, n): ...
    @property
    def opacity(self): ...
    @opacity.setter
    def opacity(self, o): ...
    def _update_opacity(self): ...
    def _set_clipper(self, node, clipper): ...
    @property
    def clip_children(self): ...
    @clip_children.setter
    def clip_children(self, clip): ...
    @property
    def clipper(self): ...
    @property
    def order(self): ...
    @order.setter
    def order(self, o): ...
    @property
    def children(self): ...
    @property
    def parent(self): ...
    @parent.setter
    def parent(self, parent): ...
    def _add_child(self, node): ...
    def _remove_child(self, node): ...
    def on_parent_change(self, event: Event): ...
    def is_child(self, node: Node) -> bool: ...
    @property
    def canvas(self): ...
    @property
    def document_node(self): ...
    @document_node.setter
    def document_node(self, doc): ...
    @property
    def scene_node(self): ...
    @property
    def root_node(self): ...
    def _set_canvas(self, c): ...
    def update(self): ...
    @property
    def document(self): ...
    @document.setter
    def document(self, doc): ...
    @property
    def transform(self): ...
    @transform.setter
    def transform(self, tr): ...
    def set_transform(self, type_: str, *args, **kwargs): ...
    def _update_trsys(self, event): ...
    def parent_chain(self): ...
    def describe_tree(self, with_transform: bool = False) -> str: ...
    def _describe_tree(self, prefix, with_transform): ...
    def common_parent(self, node: Node) -> None | Node: ...
    def node_path_to_child(self, node: Node) -> list | None: ...
    def node_path(self, node: Node) -> tuple[ArrayLike, ArrayLike]: ...
    def node_path_transforms(self, node: Node) -> ArrayLike: ...
    def node_transform(self, node: Node) -> ChainTransform: ...
    def __repr__(self): ...
    @property
    def picking(self): ...
    @picking.setter
    def picking(self, p): ...



from typing import Any, Callable, Literal, Sequence, Type
from ..base import BaseEstimator
from ..exceptions import (
    PositiveSpectrumWarning as PositiveSpectrumWarning,
    NotFittedError as NotFittedError,
    DataConversionWarning as DataConversionWarning,
)
from scipy.sparse._coo import coo_matrix
from numpy.random.mtrand import RandomState
from ..utils._array_api import get_namespace as get_namespace
from inspect import signature as signature, isclass as isclass, Parameter as Parameter
from contextlib import suppress as suppress
from numpy.core.numeric import ComplexWarning as ComplexWarning
from joblib.memory import Memory
from ._isfinite import cy_isfinite as cy_isfinite, FiniteStatus as FiniteStatus
from scipy.sparse import spmatrix
from .. import get_config as _get_config
from numpy import ndarray
from numbers import Real, Integral, Number
from functools import wraps as wraps
from pandas.api.types import is_sparse as is_sparse
from pandas import DataFrame
from .._typing import MatrixLike, ArrayLike, Int, Float
import warnings
import numbers
import operator

import numpy as np
import scipy.sparse as sp
import joblib

FLOAT_DTYPES = ...


def assert_all_finite(
    X: MatrixLike | ArrayLike,
    *,
    allow_nan: bool = False,
    estimator_name: None | str = None,
    input_name: str = "",
) -> None:
    ...


def as_float_array(
    X: MatrixLike | ArrayLike, *, copy: bool = True, force_all_finite: str | bool = True
) -> ndarray | spmatrix:
    ...


def check_memory(memory: None | Memory | str) -> Memory:
    ...


def check_consistent_length(*arrays) -> None:
    ...


def indexable(*iterables) -> None | list[ndarray | spmatrix | DataFrame]:
    ...


def check_array(
    array: Any,
    accept_sparse: Sequence[str] | tuple[str, str] | list[str] | str | bool = False,
    *,
    accept_large_sparse: bool = True,
    dtype: None | Sequence[type] | Literal["numeric", "numeric"] | type = "numeric",
    order: Literal["F", "C"] | None = None,
    copy: bool = False,
    force_all_finite: str | bool = True,
    ensure_2d: bool = True,
    allow_nd: bool = False,
    ensure_min_samples: Int = 1,
    ensure_min_features: Int = 1,
    estimator: None | str | BaseEstimator = None,
    input_name: str = "",
) -> Any:
    ...


def check_X_y(
    X: MatrixLike | ArrayLike,
    y: MatrixLike | ArrayLike,
    accept_sparse: Sequence[str] | tuple[str, str] | list[str] | str | bool = False,
    *,
    accept_large_sparse: bool = True,
    dtype: None | Sequence[type] | Literal["numeric", "numeric"] | type = "numeric",
    order: Literal["F", "C"] | None = None,
    copy: bool = False,
    force_all_finite: str | bool = True,
    ensure_2d: bool = True,
    allow_nd: bool = False,
    multi_output: bool = False,
    ensure_min_samples: Int = 1,
    ensure_min_features: Int = 1,
    y_numeric: bool = False,
    estimator: None | str | BaseEstimator = None,
) -> tuple[Any, Any]:
    ...


def column_or_1d(y: ArrayLike, *, dtype=None, warn: bool = False) -> ndarray:
    ...


def check_random_state(seed: None | Int | RandomState) -> RandomState:
    ...


def has_fit_parameter(estimator: Any, parameter: str) -> bool:
    ...


def check_symmetric(
    array: coo_matrix | MatrixLike | ArrayLike,
    *,
    tol: Float = 1e-10,
    raise_warning: bool = True,
    raise_exception: bool = False,
) -> ndarray | spmatrix:
    ...


def check_is_fitted(
    estimator: BaseEstimator,
    attributes: tuple[str, ...] | None | Sequence | list[str] | str = None,
    *,
    msg: None | str = None,
    all_or_any: Callable = ...,
) -> None:
    ...


def check_non_negative(X: MatrixLike | ArrayLike, whom: str) -> None:
    ...


def check_scalar(
    x: Any,
    name: str,
    target_type: Type[Integral] | tuple | Type[Real] | type,
    *,
    min_val: None | Float = None,
    max_val: None | Float = None,
    include_boundaries: Literal["left", "right", "both", "neither", "both"] = "both",
) -> Number | Float:
    ...



from vispy.app.canvas import Canvas

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from .linear import STTransform, NullTransform
from .chain import ChainTransform
from ._util import TransformCache
from ...util.event import EventEmitter

import numpy as np

class TransformSystem(object):
    def __init__(self, canvas: Canvas | None = None, dpi: float | None = None): ...
    def _update_if_maps_changed(self, transform, map_key, new_maps): ...
    def configure(
        self,
        viewport: tuple | None = None,
        fbo_size: tuple | None = None,
        fbo_rect: tuple | None = None,
        canvas: Canvas | None = None,
    ): ...
    @property
    def canvas(self): ...
    @canvas.setter
    def canvas(self, canvas): ...
    @property
    def dpi(self): ...
    @dpi.setter
    def dpi(self, dpi): ...
    @property
    def visual_transform(self): ...
    @visual_transform.setter
    def visual_transform(self, tr): ...
    @property
    def scene_transform(self): ...
    @scene_transform.setter
    def scene_transform(self, tr): ...
    @property
    def document_transform(self): ...
    @document_transform.setter
    def document_transform(self, tr): ...
    @property
    def canvas_transform(self): ...
    @canvas_transform.setter
    def canvas_transform(self, tr): ...
    @property
    def framebuffer_transform(self): ...
    @framebuffer_transform.setter
    def framebuffer_transform(self, tr): ...
    def get_transform(self, map_from: str = "visual", map_to: str = "render"): ...
    @property
    def pixel_scale(self): ...



from numpy.typing import NDArray
import numpy as np
from scipy import ndimage as ndi

from .. import draw
from .._shared.utils import deprecate_kwarg

def square(width: int, dtype=...) -> NDArray: ...
@deprecate_kwarg(
    {"height": "ncols", "width": "nrows"},
    deprecated_version="0.18.0",
    removed_version="0.20.0",
)
def rectangle(nrows: int, ncols: int, dtype=...) -> NDArray: ...
def diamond(radius: int, dtype=...) -> NDArray: ...
def disk(radius: int, dtype=...) -> NDArray: ...
def ellipse(width: int, height: int, dtype=...) -> NDArray: ...
def cube(width: int, dtype=...) -> NDArray: ...
def octahedron(radius: int, dtype=...) -> NDArray: ...
def ball(radius: int, dtype=...) -> NDArray: ...
def octagon(m: int, n: int, dtype=...) -> NDArray: ...
def star(a: int, dtype=...) -> NDArray: ...
def _default_footprint(ndim): ...



from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.utils.deprecation import MiddlewareMixin

class MessageMiddleware(MiddlewareMixin):
    def process_request(self, request: HttpRequest) -> None: ...
    def process_response(
        self, request: HttpRequest, response: HttpResponse
    ) -> HttpResponse: ...



from typing import Any, Callable

from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.urls.resolvers import URLResolver

def convert_exception_to_response(
    get_response: Callable[..., Any]
) -> Callable[..., Any]: ...
def response_for_exception(request: HttpRequest, exc: Exception) -> HttpResponse: ...
def get_exception_response(
    request: HttpRequest,
    resolver: URLResolver,
    status_code: int,
    exception: Exception,
    sender: None = ...,
) -> HttpResponse: ...
def handle_uncaught_exception(request: Any, resolver: Any, exc_info: Any) -> Any: ...



from ...classes.graph import Graph

__all__ = ["node_attribute_xy", "node_degree_xy"]

def node_attribute_xy(G: Graph, attribute, nodes=None): ...
def node_degree_xy(G: Graph, x="out", y="in", weight=None, nodes=None): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import struct

from .base_filter import Filter

class PickingFilter(Filter):

    FRAG_SHADER: str = ...

    def __init__(self, id_=None): ...
    @property
    def id(self): ...
    @id.setter
    def id(self, id): ...
    @property
    def enabled(self): ...
    @enabled.setter
    def enabled(self, e): ...
    @property
    def color(self): ...



from numpy.typing import ArrayLike
import numpy as np
from scipy.signal import fftconvolve

from .._shared.utils import check_nD, _supported_float_type

def _window_sum_2d(image, window_shape): ...
def _window_sum_3d(image, window_shape): ...
def match_template(
    image, template, pad_input: bool = False, mode="constant", constant_values=0
) -> ArrayLike: ...



import ctypes
from OpenGL import GL
import OpenGL.GL.framebufferobjects as FBO

def glBindAttribLocation(program, index, name): ...
def glBufferData(target, data, usage): ...
def glBufferSubData(target, offset, data): ...
def glCompressedTexImage2D(target, level, internalformat, width, height, border, data): ...
def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, data): ...
def glDeleteBuffer(buffer): ...
def glDeleteFramebuffer(framebuffer): ...
def glDeleteRenderbuffer(renderbuffer): ...
def glDeleteTexture(texture): ...
def glDrawElements(mode, count, type, offset): ...
def glCreateBuffer(): ...
def glCreateFramebuffer(): ...
def glCreateRenderbuffer(): ...
def glCreateTexture(): ...
def glGetActiveAttrib(program, index): ...
def glGetActiveUniform(program, index): ...
def glGetAttribLocation(program, name): ...
def glGetFramebufferAttachmentParameter(target, attachment, pname): ...
def glGetProgramInfoLog(program): ...
def glGetRenderbufferParameter(target, pname): ...
def glGetShaderInfoLog(shader): ...
def glGetShaderSource(shader): ...
def glGetParameter(pname): ...
def glGetUniform(program, location): ...
def glGetUniformLocation(program, name): ...
def glGetVertexAttrib(index, pname): ...
def glGetVertexAttribOffset(index, pname): ...
def glShaderSource(shader, source): ...
def glTexImage2D(target, level, internalformat, format, type, pixels): ...
def glTexSubImage2D(target, level, xoffset, yoffset, format, type, pixels): ...
def glVertexAttribPointer(indx, size, type, normalized, stride, offset): ...

# List of functions that we should import from OpenGL.GL
_functions_to_import: list = ...

# List of functions in OpenGL.GL that we use
_used_functions: list = ...



from typing import Any

from itertools import chain

from ..classes.graph import Graph
from .components import is_connected, is_strongly_connected
from .shortest_paths import shortest_path_length as spl

__all__ = ["wiener_index"]

#: Rename the :func:`chain.from_iterable` function for the sake of
#: brevity.
chaini = ...

def wiener_index(G: Graph, weight: Any = None) -> float: ...



import numpy as np
from scipy import ndimage as ndi

def _validate_connectivity(image_dim, connectivity, offset): ...
def _raveled_offsets_and_distances(
    image_shape,
    *,
    footprint=None,
    connectivity=1,
    center=None,
    spacing=None,
    order="C",
): ...
def _offsets_to_raveled_neighbors(image_shape, footprint, center, order="C"): ...
def _resolve_neighborhood(footprint, connectivity, ndim): ...
def _set_border_values(image, value): ...



from numpy.typing import ArrayLike
from copy import deepcopy
from ...classes.graph import Graph

from ...utils.decorators import not_implemented_for

__all__ = [
    "group_betweenness_centrality",
    "group_closeness_centrality",
    "group_degree_centrality",
    "group_in_degree_centrality",
    "group_out_degree_centrality",
    "prominent_group",
]

def group_betweenness_centrality(
    G: Graph, C, normalized=True, weight=None, endpoints=False
): ...
def prominent_group(
    G: Graph,
    k: int,
    weight=None,
    C=None,
    endpoints=False,
    normalized=True,
    greedy=False,
) -> tuple[float, ArrayLike]: ...
def group_closeness_centrality(G: Graph, S: ArrayLike | set, weight=None) -> float: ...
def group_degree_centrality(G: Graph, S: ArrayLike | set) -> float: ...
def group_in_degree_centrality(G: Graph, S: ArrayLike | set) -> float: ...
def group_out_degree_centrality(G: Graph, S: ArrayLike | set) -> float: ...



from typing import Mapping
from .flow_matrix import (
    CGInverseLaplacian,
    FullInverseLaplacian,
    SuperLUInverseLaplacian,
)
from ...classes.graph import Graph
from ...utils import not_implemented_for, reverse_cuthill_mckee_ordering

__all__ = ["current_flow_closeness_centrality", "information_centrality"]

def current_flow_closeness_centrality(
    G: Graph, weight=None, dtype=..., solver="lu"
) -> Mapping: ...

information_centrality = ...



# This work was originally coded by Christopher Ellison
# as part of the Computational Mechanics Python (CMPy) project.
# James P. Crutchfield, principal investigator.
# Complexity Sciences Center and Physics Department, UC Davis.

import sys
from ...classes.graph import Graph

__all__ = ["GraphMatcher", "DiGraphMatcher"]

class GraphMatcher:
    def __init__(self, G1: Graph, G2: Graph): ...
    def reset_recursion_limit(self): ...
    def candidate_pairs_iter(self): ...
    def initialize(self): ...
    def is_isomorphic(self): ...
    def isomorphisms_iter(self): ...
    def match(self): ...
    def semantic_feasibility(self, G1_node, G2_node): ...
    def subgraph_is_isomorphic(self): ...
    def subgraph_is_monomorphic(self): ...

    #    subgraph_is_isomorphic.__doc__ += "\n" + subgraph.replace('\n','\n'+indent)

    def subgraph_isomorphisms_iter(self): ...
    def subgraph_monomorphisms_iter(self): ...

    #    subgraph_isomorphisms_iter.__doc__ += "\n" + subgraph.replace('\n','\n'+indent)

    def syntactic_feasibility(self, G1_node, G2_node): ...

class DiGraphMatcher(GraphMatcher):
    def __init__(self, G1: Graph, G2: Graph): ...
    def candidate_pairs_iter(self): ...
    def initialize(self): ...
    def syntactic_feasibility(self, G1_node, G2_node): ...

class GMState:
    def __init__(self, GM, G1_node=None, G2_node=None): ...
    def restore(self): ...

class DiGMState:
    def __init__(self, GM, G1_node=None, G2_node=None): ...
    def restore(self): ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from ._base import MissingIndicator
from ..utils import is_scalar_nan as is_scalar_nan
from ._base import _BaseImputer
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral
from .._typing import Int, MatrixLike, ArrayLike
from ..metrics import pairwise_distances_chunked as pairwise_distances_chunked
from ..utils.validation import (
    FLOAT_DTYPES as FLOAT_DTYPES,
    check_is_fitted as check_is_fitted,
)

KNNImputer_Self = TypeVar("KNNImputer_Self", bound="KNNImputer")

# Authors: Ashim Bhattarai <ashimb9@gmail.com>
#          Thomas J Fan <thomasjpfan@gmail.com>
# License: BSD 3 clause

import numpy as np


class KNNImputer(_BaseImputer):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    indicator_: MissingIndicator = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        missing_values: float | None | str | int = ...,
        n_neighbors: Int = 5,
        weights: Literal["uniform", "distance", "uniform"] | Callable = "uniform",
        metric: Callable | Literal["nan_euclidean", "nan_euclidean"] = "nan_euclidean",
        copy: bool = True,
        add_indicator: bool = False,
        keep_empty_features: bool = False,
    ) -> None:
        ...

    def fit(
        self: KNNImputer_Self, X: list[list[int | float]] | MatrixLike, y: Any = None
    ) -> KNNImputer_Self:
        ...

    def transform(self, X: list[list[int | float]] | MatrixLike) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...



import sys
import warnings

from ..classes.graph import Graph
from ..algorithms.components import connected_components
from ..exception import NetworkXException
from ..utils import arbitrary_element, not_implemented_for

__all__ = [
    "is_chordal",
    "find_induced_nodes",
    "chordal_graph_cliques",
    "chordal_graph_treewidth",
    "NetworkXTreewidthBoundExceeded",
    "complete_to_chordal_graph",
]

class NetworkXTreewidthBoundExceeded(NetworkXException):
    pass

def is_chordal(G: Graph) -> bool: ...
def find_induced_nodes(G: Graph, s, t, treewidth_bound: float = ...): ...
def chordal_graph_cliques(G: Graph): ...
def chordal_graph_treewidth(G: Graph) -> int: ...
def complete_to_chordal_graph(G: Graph): ...



"""
This type stub file was generated by pyright.
"""

import codeop
from contextlib import contextmanager

"""Compiler tools with improved interactive support.

Provides compilation machinery similar to codeop, but with caching support so
we can provide interactive tracebacks.

Authors
-------
* Robert Kern
* Fernando Perez
* Thomas Kluyver
"""
PyCF_MASK = ...
def code_name(code, number=...):
    """ Compute a (probably) unique name for code for caching.

    This now expects code to be unicode.
    """
    ...

class CachingCompiler(codeop.Compile):
    """A compiler that caches code compiled from interactive statements.
    """
    def __init__(self) -> None:
        ...
    
    def ast_parse(self, source, filename=..., symbol=...):
        """Parse code to an AST with the current compiler flags active.

        Arguments are exactly the same as ast.parse (in the standard library),
        and are passed to the built-in compile function."""
        ...
    
    def reset_compiler_flags(self): # -> None:
        """Reset compiler flags to default state."""
        ...
    
    @property
    def compiler_flags(self):
        """Flags currently active in the compilation process.
        """
        ...
    
    def get_code_name(self, raw_code, transformed_code, number):
        """Compute filename given the code, and the cell number.

        Parameters
        ----------
        raw_code : str
            The raw cell code.
        transformed_code : str
            The executable Python source code to cache and compile.
        number : int
            A number which forms part of the code's name. Used for the execution
            counter.

        Returns
        -------
        The computed filename.
        """
        ...
    
    def format_code_name(self, name): # -> None:
        """Return a user-friendly label and name for a code block.

        Parameters
        ----------
        name : str
            The name for the code block returned from get_code_name

        Returns
        -------
        A (label, name) pair that can be used in tracebacks, or None if the default formatting should be used.
        """
        ...
    
    def cache(self, transformed_code, number=..., raw_code=...):
        """Make a name for a block of code, and cache the code.

        Parameters
        ----------
        transformed_code : str
            The executable Python source code to cache and compile.
        number : int
            A number which forms part of the code's name. Used for the execution
            counter.
        raw_code : str
            The raw code before transformation, if None, set to `transformed_code`.

        Returns
        -------
        The name of the cached code (as a string). Pass this as the filename
        argument to compilation, so that tracebacks are correctly hooked up.
        """
        ...
    
    @contextmanager
    def extra_flags(self, flags):
        ...
    


def check_linecache_ipython(*args): # -> None:
    """Deprecated since IPython 8.6.  Call linecache.checkcache() directly.

    It was already not necessary to call this function directly.  If no
    CachingCompiler had been created, this function would fail badly.  If
    an instance had been created, this function would've been monkeypatched
    into place.

    As of IPython 8.6, the monkeypatching has gone away entirely.  But there
    were still internal callers of this function, so maybe external callers
    also existed?
    """
    ...




from ..classes.graph import Graph
from ..classes.graph import Graph
from ..exception import NetworkXError
from ..utils import py_random_state

__all__ = ["partial_duplication_graph", "duplication_divergence_graph"]

@py_random_state(4)
def partial_duplication_graph(N: int, n: int, p: float, q: float, seed=None): ...
@py_random_state(2)
def duplication_divergence_graph(n: int, p: float, seed=None) -> Graph: ...



from collections.abc import Iterable
from collections import Counter, defaultdict

from ..classes.graph import Graph

__all__ = ["dedensify", "snap_aggregation"]

def dedensify(G: Graph, threshold: int, prefix=None, copy=True): ...
def snap_aggregation(
    G: Graph,
    node_attributes,
    edge_attributes: Iterable = ...,
    prefix: str = "Supernode-",
    supernode_attribute: str = "group",
    superedge_attribute: str = "types",
): ...



from numpy.typing import NDArray
from typing import Literal

import numpy as np

from scipy import ndimage as ndi

from .._shared.utils import check_nD, deprecate_kwarg

def skeletonize(image, *, method: Literal["zhang", "lee"] | None = None) -> NDArray: ...
def skeletonize_2d(image) -> NDArray: ...

# --------- Skeletonization and thinning based on Guo and Hall 1989 ---------

def _generate_thin_luts(): ...

G123_LUT = ...

G123P_LUT = ...

@deprecate_kwarg(
    {"max_iter": "max_num_iter"}, removed_version="1.0", deprecated_version="0.19"
)
def thin(image, max_num_iter=None) -> NDArray: ...

# --------- Skeletonization by medial axis transform --------

_eight_connect = ...

def medial_axis(
    image, mask=None, return_distance: bool = False, *, random_state=None
) -> tuple[NDArray, NDArray]: ...
def _pattern_of(index): ...
def _table_lookup(image, table): ...
def skeletonize_3d(image) -> NDArray: ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable

"""
System command aliases.

Authors:

* Fernando Perez
* Brian Granger
"""
shell_line_split = ...
def default_aliases():
    """Return list of shell aliases to auto-define.
    """
    ...

class AliasError(Exception):
    ...


class InvalidAliasError(AliasError):
    ...


class Alias:
    """Callable object storing the details of one alias.

    Instances are registered as magic functions to allow use of aliases.
    """
    blacklist = ...
    def __init__(self, shell, name, cmd) -> None:
        ...
    
    def validate(self):
        """Validate the alias, and return the number of arguments."""
        ...
    
    def __repr__(self):
        ...
    
    def __call__(self, rest=...): # -> None:
        ...
    


class AliasManager(Configurable):
    default_aliases = ...
    user_aliases = ...
    shell = ...
    def __init__(self, shell=..., **kwargs) -> None:
        ...
    
    def init_aliases(self): # -> None:
        ...
    
    @property
    def aliases(self):
        ...
    
    def soft_define_alias(self, name, cmd): # -> None:
        """Define an alias, but don't raise on an AliasError."""
        ...
    
    def define_alias(self, name, cmd): # -> None:
        """Define a new alias after validating it.

        This will raise an :exc:`AliasError` if there are validation
        problems.
        """
        ...
    
    def get_alias(self, name): # -> None:
        """Return an alias, or None if no alias by that name exists."""
        ...
    
    def is_alias(self, name):
        """Return whether or not a given name has been defined as an alias"""
        ...
    
    def undefine_alias(self, name): # -> None:
        ...
    
    def clear_aliases(self): # -> None:
        ...
    
    def retrieve_alias(self, name):
        """Retrieve the command to which an alias expands."""
        ...
    





from pandas.core.indexes.extension import ExtensionIndex
from pandas.core.indexes.timedeltas import TimedeltaIndex

from pandas._libs.tslibs import BaseOffset

class DatetimeIndexOpsMixin(ExtensionIndex):
    @property
    def freq(self) -> BaseOffset | None: ...
    @property
    def freqstr(self) -> str | None: ...
    @property
    def is_all_dates(self) -> bool: ...
    def min(self, axis=..., skipna: bool = ..., *args, **kwargs): ...
    def argmin(self, axis=..., skipna: bool = ..., *args, **kwargs): ...
    def max(self, axis=..., skipna: bool = ..., *args, **kwargs): ...
    def argmax(self, axis=..., skipna: bool = ..., *args, **kwargs): ...
    def __rsub__(self, other: DatetimeIndexOpsMixin) -> TimedeltaIndex: ...

class DatetimeTimedeltaMixin(DatetimeIndexOpsMixin): ...



from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = [
    "uniform_random_intersection_graph",
    "k_random_intersection_graph",
    "general_random_intersection_graph",
]

@py_random_state(3)
def uniform_random_intersection_graph(n: int, m: int, p: float, seed=None): ...
@py_random_state(3)
def k_random_intersection_graph(n: int, m: int, k: float, seed=None): ...
@py_random_state(3)
def general_random_intersection_graph(n: int, m: int, p, seed=None): ...



"""
This type stub file was generated by pyright.
"""

from typing import Any, Callable, Dict, List, Union

"""Callbacks library."""
class EarlyStopException(Exception):
    """Exception of early stopping."""
    def __init__(self, best_iteration: int, best_score: float) -> None:
        """Create early stopping exception.

        Parameters
        ----------
        best_iteration : int
            The best iteration stopped.
        best_score : float
            The score of the best iteration.
        """
        ...
    


CallbackEnv = ...
def print_evaluation(period: int = ..., show_stdv: bool = ...) -> Callable:
    """Create a callback that logs the evaluation results.

    Deprecated, use ``log_evaluation()`` instead.
    """
    ...

def log_evaluation(period: int = ..., show_stdv: bool = ...) -> Callable:
    """Create a callback that logs the evaluation results.

    By default, standard output resource is used.
    Use ``register_logger()`` function to register a custom logger.

    Note
    ----
    Requires at least one validation data.

    Parameters
    ----------
    period : int, optional (default=1)
        The period to log the evaluation results.
        The last boosting stage or the boosting stage found by using ``early_stopping`` callback is also logged.
    show_stdv : bool, optional (default=True)
        Whether to log stdv (if provided).

    Returns
    -------
    callback : callable
        The callback that logs the evaluation results every ``period`` boosting iteration(s).
    """
    ...

def record_evaluation(eval_result: Dict[str, Dict[str, List[Any]]]) -> Callable:
    """Create a callback that records the evaluation history into ``eval_result``.

    Parameters
    ----------
    eval_result : dict
        Dictionary used to store all evaluation results of all validation sets.
        This should be initialized outside of your call to ``record_evaluation()`` and should be empty.
        Any initial contents of the dictionary will be deleted.

        .. rubric:: Example

        With two validation sets named 'eval' and 'train', and one evaluation metric named 'logloss'
        this dictionary after finishing a model training process will have the following structure:

        .. code-block::

            {
             'train':
                 {
                  'logloss': [0.48253, 0.35953, ...]
                 },
             'eval':
                 {
                  'logloss': [0.480385, 0.357756, ...]
                 }
            }

    Returns
    -------
    callback : callable
        The callback that records the evaluation history into the passed dictionary.
    """
    ...

def reset_parameter(**kwargs: Union[list, Callable]) -> Callable:
    """Create a callback that resets the parameter after the first iteration.

    .. note::

        The initial parameter will still take in-effect on first iteration.

    Parameters
    ----------
    **kwargs : value should be list or callable
        List of parameters for each boosting round
        or a callable that calculates the parameter in terms of
        current number of round (e.g. yields learning rate decay).
        If list lst, parameter = lst[current_round].
        If callable func, parameter = func(current_round).

    Returns
    -------
    callback : callable
        The callback that resets the parameter after the first iteration.
    """
    ...

def early_stopping(stopping_rounds: int, first_metric_only: bool = ..., verbose: bool = ...) -> Callable:
    """Create a callback that activates early stopping.

    Activates early stopping.
    The model will train until the validation score stops improving.
    Validation score needs to improve at least every ``stopping_rounds`` round(s)
    to continue training.
    Requires at least one validation data and one metric.
    If there's more than one, will check all of them. But the training data is ignored anyway.
    To check only the first metric set ``first_metric_only`` to True.
    The index of iteration that has the best performance will be saved in the ``best_iteration`` attribute of a model.

    Parameters
    ----------
    stopping_rounds : int
        The possible number of rounds without the trend occurrence.
    first_metric_only : bool, optional (default=False)
        Whether to use only the first metric for early stopping.
    verbose : bool, optional (default=True)
        Whether to log message with early stopping information.
        By default, standard output resource is used.
        Use ``register_logger()`` function to register a custom logger.

    Returns
    -------
    callback : callable
        The callback that activates early stopping.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import os
from IPython.utils.coloransi import TermColors
from .colorable import Colorable

"""
Class and program to colorize python source code for ANSI terminals.

Based on an HTML code highlighter by Jurgen Hermann found at:
http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52298

Modifications by Fernando Perez (fperez@colorado.edu).

Information on the original HTML highlighter follows:

MoinMoin - Python Source Parser

Title: Colorize Python source using the built-in tokenizer

Submitter: Jurgen Hermann
Last Updated:2001/04/06

Version no:1.2

Description:

This code is part of MoinMoin (http://moin.sourceforge.net/) and converts
Python source code to HTML markup, rendering comments, keywords,
operators, numeric and string literals in different colors.

It shows how to use the built-in keyword, token and tokenize modules to
scan Python source code and re-emit it with no changes to its original
formatting (which is the hard part).
"""
__all__ = ['ANSICodeColors', 'Parser']
_scheme_default = ...
generate_tokens = ...
_KEYWORD = ...
_TEXT = ...
Colors = TermColors
NoColor = ...
LinuxColors = ...
NeutralColors = ...
if os.name == 'nt':
    NeutralColors = ...
LightBGColors = ...
ANSICodeColors = ...
Undefined = ...
class Parser(Colorable):
    """ Format colored Python source.
    """
    def __init__(self, color_table=..., out=..., parent=..., style=...) -> None:
        """ Create a parser with a specified color table and output channel.

        Call format() to process code.
        """
        ...
    
    def format(self, raw, out=..., scheme=...):
        ...
    
    def format2(self, raw, out=...):
        """ Parse and send the colored source.

        If out and scheme are not specified, the defaults (given to
        constructor) are used.

        out should be a file-type object. Optionally, out can be given as the
        string 'str' and the parser will automatically return the output in a
        string."""
        ...
    
    def __call__(self, toktype, toktext, start_pos, end_pos, line): # -> None:
        """ Token handler, with syntax highlighting."""
        ...
    






import numpy as np
from ._criterion import Criterion


class SplitRecord:
    # Data to track sample split
    feature: int         # Which feature to split on.
    pos: int             # Split samples array at the given position,
                           # i.e. count of samples below threshold for feature.
                           # pos is >= end if the node is a leaf.
    threshold: float       # Threshold to split at.
    improvement: float     # Impurity improvement given parent node.
    impurity_left: float   # Impurity of the left split.
    impurity_right: float  # Impurity of the right split.


class Splitter:
    """ The splitter searches in the input space for a feature and a threshold
        to split the samples samples[start:end]. """

    criterion: Criterion      # Impurity criterion
    max_features: int         # Number of features to test
    min_samples_leaf: int     # Min samples in a leaf
    min_weight_leaf: float    # Minimum weight in a leaf

    random_state: object             # Random state
    rand_r_state: int           # sklearn_rand_r random number state

    samples: np.ndarray
    n_samples: int
    weighted_n_samples: float
    features: np.ndarray
    constant_features: np.ndarray
    n_features: int
    feature_values: np.ndarray

    start: int
    end: int

    y: np.ndarray
    sample_weight: np.ndarray

    def init(
        self,
        X: object,
        y: np.ndarray,
        sample_weight: np.ndarray
    ) -> int: ...

    def node_reset(
        self,
        start: int,
        end: int,
        weighted_n_node_samples: list[float]
    ) -> int: ...

    def node_split(
        self,
        impurity: float,
        split: SplitRecord,
        n_constant_features: list[int]
    ) -> int: ...

    def node_value(self, dest: list[float]) -> None: ...
    def node_impurity(self) -> float: ...



"""
This type stub file was generated by pyright.
"""

"""String dispatch class to match regexps and dispatch commands.
"""
class StrDispatch:
    """Dispatch (lookup) a set of strings / regexps for match.

    Example:

    >>> dis = StrDispatch()
    >>> dis.add_s('hei',34, priority = 4)
    >>> dis.add_s('hei',123, priority = 2)
    >>> dis.add_re('h.i', 686)
    >>> print(list(dis.flat_matches('hei')))
    [123, 34, 686]
    """
    def __init__(self) -> None:
        ...
    
    def add_s(self, s, obj, priority=...): # -> None:
        """ Adds a target 'string' for dispatching """
        ...
    
    def add_re(self, regex, obj, priority=...): # -> None:
        """ Adds a target regexp for dispatching """
        ...
    
    def dispatch(self, key):
        """ Get a seq of Commandchain objects that match key """
        ...
    
    def __repr__(self):
        ...
    
    def s_matches(self, key):
        ...
    
    def flat_matches(self, key):
        """ Yield all 'value' targets, without priority """
        ...
    





from typing import Any, List, Optional, Type, Union

from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.db.models.options import Options
from django.http.request import HttpRequest
from django.test.client import Client

from .signals import user_logged_in as user_logged_in
from .signals import user_logged_out as user_logged_out
from .signals import user_login_failed as user_login_failed

SESSION_KEY: str
BACKEND_SESSION_KEY: str
HASH_SESSION_KEY: str
REDIRECT_FIELD_NAME: str

def load_backend(path: str) -> ModelBackend: ...
def get_backends() -> List[ModelBackend]: ...
def authenticate(
    request: Any = ..., **credentials: Any
) -> Optional[AbstractBaseUser]: ...
def login(
    request: HttpRequest,
    user: Optional[AbstractBaseUser],
    backend: Optional[Union[Type[ModelBackend], str]] = ...,
) -> None: ...
def logout(request: HttpRequest) -> None: ...
def get_user_model() -> Type[AbstractBaseUser]: ...
def get_user(
    request: Union[HttpRequest, Client]
) -> Union[AbstractBaseUser, AnonymousUser]: ...
def get_permission_codename(action: str, opts: Options[Any]) -> str: ...
def update_session_auth_hash(request: HttpRequest, user: AbstractBaseUser) -> None: ...

default_app_config: str



import numpy as np
from typing import Callable, Literal, Sequence
from ._typing import *

def window_hanning(x): ...
def window_none(x): ...
def detrend(
    x: Sequence,
    key: Literal["default", "constant", "mean", "linear", "none"] | Callable = ...,
    axis: int = ...,
) -> Sequence: ...
def detrend_mean(x: Sequence, axis: int = ...) -> Sequence: ...
def detrend_none(x, axis: int = ...): ...
def detrend_linear(y: Sequence) -> Sequence: ...
def stride_windows(x: Sequence, n: int, noverlap: int = ..., axis: int = ...): ...
def psd(
    x: Sequence,
    NFFT: int = ...,
    Fs: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: int = 0,
    pad_to: int = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
) -> tuple[np.ndarray, np.ndarray]: ...
def csd(
    x: ArrayLike,
    y: ArrayLike,
    NFFT: int = ...,
    Fs: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: int = 0,
    pad_to: int = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
) -> tuple[np.ndarray, np.ndarray]: ...

complex_spectrum = ...
magnitude_spectrum = ...
angle_spectrum = ...
phase_spectrum = ...

def specgram(
    x: ArrayLike,
    NFFT: int = ...,
    Fs: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: int = 0,
    pad_to: int = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
    mode: str = ...,
) -> tuple[ArrayLike, ArrayLike, ArrayLike]: ...
def cohere(
    x,
    y,
    NFFT: int = ...,
    Fs: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: int = 0,
    pad_to: int = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
) -> tuple[np.ndarray, np.ndarray]: ...

class GaussianKDE:
    def __init__(
        self, dataset: ArrayLike, bw_method: str | Scalar | Callable = ...
    ) -> None: ...
    def scotts_factor(self): ...
    def silverman_factor(self): ...

    covariance_factor = ...
    def evaluate(self, points) -> np.ndarray: ...



from enum import Enum
from typing import Any, Optional, Sequence, Tuple, Type, TypeVar, Union

from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.models.base import Model
from django.db.models.expressions import BaseExpression, Combinable
from django.db.models.query_utils import Q

_T = TypeVar("_T", bound="BaseConstraint")

class Deferrable(Enum):
    DEFERRED: str
    IMMEDIATE: str

class BaseConstraint:
    name: str
    def __init__(
        self,
        *args: BaseExpression | Combinable | str,
        name: Optional[str] = ...,
        violation_error_message: Optional[str] = ...,
    ) -> None: ...
    def constraint_sql(
        self,
        model: Optional[Type[Model]],
        schema_editor: Optional[BaseDatabaseSchemaEditor],
    ) -> str: ...
    def create_sql(
        self,
        model: Optional[Type[Model]],
        schema_editor: Optional[BaseDatabaseSchemaEditor],
    ) -> str: ...
    def remove_sql(
        self,
        model: Optional[Type[Model]],
        schema_editor: Optional[BaseDatabaseSchemaEditor],
    ) -> str: ...
    def deconstruct(self) -> Any: ...
    def clone(self: _T) -> _T: ...

class CheckConstraint(BaseConstraint):
    check: Q
    def __init__(
        self,
        *,
        check: Q,
        name: str,
        violation_error_message: Optional[str] = ...,
    ) -> None: ...

class UniqueConstraint(BaseConstraint):
    fields: Tuple[str]
    condition: Optional[Q]
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        fields: Sequence[str] = ...,
        name: Optional[str] = ...,
        condition: Optional[Q] = ...,
        deferrable: Optional[Deferrable] = ...,
        include: Optional[Union[str, Sequence[str]]] = ...,
        opclasses: Sequence[str] = ...,
        violation_error_message: Optional[str] = ...,
    ) -> None: ...



"""
This type stub file was generated by pyright.
"""

"""IO capturing utilities."""
class RichOutput:
    def __init__(self, data=..., metadata=..., transient=..., update=...) -> None:
        ...
    
    def display(self): # -> None:
        ...
    


class CapturedIO:
    """Simple object for containing captured stdout/err and rich display StringIO objects

    Each instance `c` has three attributes:

    - ``c.stdout`` : standard output as a string
    - ``c.stderr`` : standard error as a string
    - ``c.outputs``: a list of rich display outputs

    Additionally, there's a ``c.show()`` method which will print all of the
    above in the same order, and can be invoked simply via ``c()``.
    """
    def __init__(self, stdout, stderr, outputs=...) -> None:
        ...
    
    def __str__(self) -> str:
        ...
    
    @property
    def stdout(self):
        "Captured standard output"
        ...
    
    @property
    def stderr(self):
        "Captured standard error"
        ...
    
    @property
    def outputs(self):
        """A list of the captured rich display outputs, if any.

        If you have a CapturedIO object ``c``, these can be displayed in IPython
        using::

            from IPython.display import display
            for o in c.outputs:
                display(o)
        """
        ...
    
    def show(self): # -> None:
        """write my output to sys.stdout/err as appropriate"""
        ...
    
    __call__ = ...


class capture_output:
    """context manager for capturing stdout/err"""
    stdout = ...
    stderr = ...
    display = ...
    def __init__(self, stdout=..., stderr=..., display=...) -> None:
        ...
    
    def __enter__(self): # -> CapturedIO:
        ...
    
    def __exit__(self, exc_type, exc_value, traceback): # -> None:
        ...
    





import pathlib
from matplotlib.path import Path
from typing import Any, Callable, Literal, Sequence
from ._typing import *
from .transforms import Bbox, Transform
from .backend_bases import MouseEvent, RendererBase
from .font_manager import FontProperties
from .figure import Figure
from .artist import Artist, allow_rasterization
from .offsetbox import DraggableAnnotation
from .patches import FancyBboxPatch
from pathlib import PosixPath

def get_rotation(
    rotation: float | Literal[None, "horizontal", "vertical"]
) -> float: ...

class Text(Artist):

    zorder = ...
    def __repr__(self): ...
    def __init__(
        self,
        x: float = ...,
        y: float = ...,
        text: str = ...,
        color: Color = ...,
        verticalalignment: str = ...,
        horizontalalignment: str = ...,
        multialignment: str = ...,
        fontproperties: FontProperties = ...,
        rotation: float | str = ...,
        linespacing: float = ...,
        rotation_mode: str = ...,
        usetex: bool = ...,
        wrap: bool = ...,
        transform_rotates_text: bool = ...,
        *,
        parse_math=...,
        **kwargs
    ) -> None: ...
    def update(self, kwargs: dict[str, Any]) -> None: ...
    def __getstate__(self): ...
    def contains(self, mouseevent: MouseEvent) -> tuple[bool, dict]: ...
    def get_rotation(self) -> float: ...
    def get_transform_rotates_text(self) -> bool: ...
    def set_rotation_mode(self, m: None | Literal["default", "anchor"]) -> None: ...
    def get_rotation_mode(self) -> None | str: ...
    def update_from(self, other: Text) -> None: ...
    def set_bbox(self, rectprops: dict[str, Any]) -> None: ...
    def get_bbox_patch(self) -> None | FancyBboxPatch: ...
    def update_bbox_position_size(self, renderer: RendererBase) -> None: ...
    def set_clip_box(self, clipbox) -> None: ...
    def set_clip_path(self, path: Path, transform: Transform = ...) -> None: ...
    def set_clip_on(self, b: bool) -> None: ...
    def get_wrap(self) -> bool: ...
    def set_wrap(self, wrap: bool) -> None: ...
    @allow_rasterization
    def draw(self, renderer): ...
    def get_color(
        self,
    ) -> Color: ...
    def get_fontproperties(self) -> FontProperties: ...
    def get_fontfamily(self) -> list[str]: ...
    def get_fontname(self) -> str: ...
    def get_fontstyle(self) -> str: ...
    def get_fontsize(self) -> float: ...
    def get_fontvariant(self) -> str: ...
    def get_fontweight(self) -> str: ...
    def get_stretch(self) -> str: ...
    def get_horizontalalignment(self) -> str: ...
    def get_unitless_position(self) -> tuple[float, float]: ...
    def get_position(
        self,
    ) -> tuple[float, float]: ...
    def get_prop_tup(self, renderer: RendererBase = ...): ...
    def get_text(self) -> str: ...
    def get_verticalalignment(self) -> str: ...
    def get_window_extent(
        self,
        renderer: RendererBase = ...,
        dpi: float = ...,
    ): ...
    def set_backgroundcolor(self, color: Color) -> None: ...
    def set_color(self, color: Color) -> None: ...
    def set_horizontalalignment(
        self, align: Literal["left", "right", "center"]
    ) -> None: ...
    def set_multialignment(self, align: Literal["left", "right", "center"]) -> None: ...
    def set_linespacing(self, spacing): ...
    def set_fontfamily(
        self,
        fontname: str
        | Literal["serif", "sans-serif", "cursive", "fantasy", "monospace"],
    ) -> None: ...
    def set_fontvariant(self, variant: Literal["normal", "small-caps"]) -> None: ...
    def set_fontstyle(
        self, fontstyle: Literal["normal", "italic", "oblique"]
    ) -> None: ...
    def set_fontsize(
        self,
        fontsize: float
        | Literal[
            "xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large"
        ],
    ) -> None: ...
    def get_math_fontfamily(self) -> str: ...
    def set_math_fontfamily(self, fontfamily: str) -> None: ...
    def set_fontweight(
        self,
        weight: float
        | Literal[
            "ultralight",
            "light",
            "normal",
            "regular",
            "book",
            "medium",
            "roman",
            "semibold",
            "demibold",
            "demi",
            "bold",
            "heavy",
            "extra bold",
            "black",
        ],
    ) -> None: ...
    def set_fontstretch(
        self,
        stretch: float
        | Literal[
            "ultra-condensed",
            "extra-condensed",
            "condensed",
            "semi-condensed",
            "normal",
            "semi-expanded",
            "expanded",
            "extra-expanded",
            "ultra-expanded",
        ],
    ): ...
    def set_position(self, xy: Sequence[float]) -> None: ...
    def set_x(self, x: float) -> None: ...
    def set_y(self, y: float) -> None: ...
    def set_rotation(self, s: float | Literal["vertical", "horizontal"]) -> None: ...
    def set_transform_rotates_text(self, t: bool): ...
    def set_verticalalignment(
        self, align: Literal["bottom", "baseline", "center", "center_baseline", "top"]
    ) -> None: ...
    def set_text(self, s) -> None: ...
    def set_fontproperties(
        self, fp: str | pathlib.Path | PosixPath | FontProperties
    ) -> None: ...
    def set_usetex(self, usetex: bool) -> None: ...
    def get_usetex(self) -> bool: ...
    def set_parse_math(self, parse_math: bool) -> None: ...
    def get_parse_math(self) -> bool: ...
    def set_fontname(
        self,
        fontname: str
        | Literal["serif", "sans-serif", "cursive", "fantasy", "monospace"],
    ) -> None: ...

class OffsetFrom:
    def __init__(
        self,
        artist: Annotation,
        ref_coord: Sequence[float],
        unit: Literal["points", "pixels"] = "points",
    ) -> None: ...
    def set_unit(self, unit: str) -> None: ...
    def get_unit(self) -> str: ...
    def __call__(self, renderer: RendererBase) -> Transform: ...

class _AnnotationBase:
    def __init__(
        self,
        xy: Sequence[float],
        xycoords: str | Artist | Transform | Callable | Sequence[float] = ...,
        annotation_clip: bool = ...,
    ) -> None: ...
    def set_annotation_clip(self, b: bool | None) -> None: ...
    def get_annotation_clip(self) -> bool | None: ...
    def draggable(
        self, state: bool | None = ..., use_blit: bool = ...
    ) -> None | DraggableAnnotation: ...

class Annotation(Text, _AnnotationBase):
    def __str__(self) -> str: ...
    def __init__(
        self,
        text: str,
        xy: Sequence[float],
        xytext: Sequence[float] = ...,
        xycoords: str | Artist | Transform | Callable | Sequence[float] = "data",
        textcoords: str | Artist | Transform | Callable | Sequence[float] = ...,
        arrowprops: dict = ...,
        annotation_clip: bool | None = ...,
        **kwargs
    ) -> None: ...
    def contains(self, event): ...
    @property
    def xycoords(self) -> str | Artist | Transform | Callable | Sequence[float]: ...
    @xycoords.setter
    def xycoords(
        self, xycoords: str | Artist | Transform | Callable | Sequence[float]
    ): ...
    @property
    def xyann(self): ...
    @xyann.setter
    def xyann(self, xytext): ...
    def get_anncoords(
        self,
    ) -> str | Artist | Transform | Callable | Sequence[float]: ...
    def set_anncoords(
        self, coords: str | Artist | Transform | Callable | Sequence[float]
    ): ...
    anncoords = ...
    def set_figure(self, fig: Figure) -> None: ...
    def update_positions(self, renderer: RendererBase) -> None: ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_tightbbox(self, renderer: RendererBase = ...) -> Bbox: ...



from ...algorithms.tree.mst import SpanningTreeIterator
from dataclasses import dataclass, field
from enum import Enum
from heapq import heappop, heappush
from itertools import count
from math import isnan
from operator import itemgetter
from queue import PriorityQueue

from ...classes.graph import Graph
from ...utils import UnionFind, not_implemented_for, py_random_state

__all__ = [
    "minimum_spanning_edges",
    "maximum_spanning_edges",
    "minimum_spanning_tree",
    "maximum_spanning_tree",
    "random_spanning_tree",
    "partition_spanning_tree",
    "EdgePartition",
    "SpanningTreeIterator",
]

class EdgePartition(Enum):

    OPEN = ...
    INCLUDED = ...
    EXCLUDED = ...

def boruvka_mst_edges(
    G: Graph, minimum=True, weight="weight", keys=False, data=True, ignore_nan=False
): ...
def kruskal_mst_edges(
    G: Graph,
    minimum,
    weight="weight",
    keys=True,
    data=True,
    ignore_nan=False,
    partition=None,
): ...
def prim_mst_edges(
    G: Graph, minimum, weight="weight", keys=True, data=True, ignore_nan=False
): ...

ALGORITHMS: dict = ...

def minimum_spanning_edges(
    G: Graph,
    algorithm: str = "kruskal",
    weight: str = "weight",
    keys: bool = True,
    data: bool = True,
    ignore_nan=False,
): ...
def maximum_spanning_edges(
    G: Graph,
    algorithm: str = "kruskal",
    weight: str = "weight",
    keys: bool = True,
    data: bool = True,
    ignore_nan=False,
): ...
def minimum_spanning_tree(
    G: Graph, weight: str = "weight", algorithm: str = "kruskal", ignore_nan=False
): ...
def partition_spanning_tree(
    G: Graph,
    minimum=True,
    weight: str = "weight",
    partition: str = "partition",
    ignore_nan=False,
): ...
def maximum_spanning_tree(
    G: Graph, weight: str = "weight", algorithm: str = "kruskal", ignore_nan=False
): ...
@py_random_state(3)
def random_spanning_tree(
    G: Graph, weight: str | None = None, *, multiplicative: bool = True, seed=None
): ...

class SpanningTreeIterator:
    ...

    def __init__(
        self,
        G: Graph,
        weight: str = "weight",
        minimum: bool = True,
        ignore_nan: bool = False,
    ): ...
    def __iter__(self) -> SpanningTreeIterator: ...
    def __next__(self): ...
    def _partition(self, partition, partition_tree): ...
    def _write_partition(self, partition): ...
    def _clear_partition(self, G): ...



from typing import Mapping
from .pairs import node_attribute_xy, node_degree_xy
from ...classes.graph import Graph
from ...utils import dict_to_numpy_array

__all__ = [
    "attribute_mixing_matrix",
    "attribute_mixing_dict",
    "degree_mixing_matrix",
    "degree_mixing_dict",
    "numeric_mixing_matrix",
    "mixing_dict",
]

def attribute_mixing_dict(
    G: Graph, attribute: str, nodes=None, normalized=False
) -> Mapping: ...
def attribute_mixing_matrix(
    G: Graph,
    attribute: str,
    nodes=None,
    mapping: Mapping | None = None,
    normalized=True,
): ...
def degree_mixing_dict(
    G: Graph, x="out", y="in", weight=None, nodes=None, normalized=False
) -> Mapping: ...
def degree_mixing_matrix(
    G: Graph,
    x="out",
    y="in",
    weight=None,
    nodes=None,
    normalized=True,
    mapping: Mapping | None = None,
): ...
def numeric_mixing_matrix(
    G: Graph,
    attribute: str,
    nodes=None,
    normalized=True,
    mapping: Mapping | None = None,
): ...
def mixing_dict(xy, normalized=False) -> Mapping: ...



from .connectivity import (
    average_degree_connectivity as average_degree_connectivity,
    k_nearest_neighbors as k_nearest_neighbors,
)
from .correlation import (
    degree_pearson_correlation_coefficient as degree_pearson_correlation_coefficient,
    degree_assortativity_coefficient as degree_assortativity_coefficient,
    attribute_assortativity_coefficient as attribute_assortativity_coefficient,
    numeric_assortativity_coefficient as numeric_assortativity_coefficient,
)
from .mixing import (
    attribute_mixing_matrix as attribute_mixing_matrix,
    attribute_mixing_dict as attribute_mixing_dict,
    degree_mixing_matrix as degree_mixing_matrix,
    degree_mixing_dict as degree_mixing_dict,
    numeric_mixing_matrix as numeric_mixing_matrix,
    mixing_dict as mixing_dict,
)
from .neighbor_degree import average_neighbor_degree as average_neighbor_degree
from .pairs import (
    node_attribute_xy as node_attribute_xy,
    node_degree_xy as node_degree_xy,
)



from collections import deque

from ...classes.graph import Graph

__all__ = [
    "bfs_edges",
    "bfs_tree",
    "bfs_predecessors",
    "bfs_successors",
    "descendants_at_distance",
    "bfs_layers",
]

def generic_bfs_edges(
    G: Graph, source, neighbors=None, depth_limit=None, sort_neighbors=None
): ...
def bfs_edges(
    G: Graph, source, reverse: bool = False, depth_limit=None, sort_neighbors=None
): ...
def bfs_tree(
    G: Graph, source, reverse: bool = False, depth_limit=None, sort_neighbors=None
): ...
def bfs_predecessors(G: Graph, source, depth_limit=None, sort_neighbors=None): ...
def bfs_successors(G: Graph, source, depth_limit=None, sort_neighbors=None): ...
def bfs_layers(G: Graph, sources): ...
def descendants_at_distance(G: Graph, source, distance): ...



from typing import Any, Union, overload

from django.utils.functional import Promise

class DjangoUnicodeDecodeError(UnicodeDecodeError):
    obj: bytes = ...
    def __init__(self, obj: bytes, *args: Any) -> None: ...

def smart_text(
    s: Any, encoding: str = ..., strings_only: bool = ..., errors: str = ...
) -> Any: ...
def is_protected_type(obj: Any) -> bool: ...
def force_text(
    s: Any, encoding: str = ..., strings_only: bool = ..., errors: str = ...
) -> Any: ...
def force_bytes(
    s: Any, encoding: str = ..., strings_only: bool = ..., errors: str = ...
) -> Any: ...

smart_str = smart_text
force_str = force_text

@overload
def iri_to_uri(iri: None) -> None: ...
@overload
def iri_to_uri(iri: Union[str, Promise]) -> str: ...
@overload
def uri_to_iri(iri: None) -> None: ...
@overload
def uri_to_iri(iri: str) -> str: ...
def escape_uri_path(path: str) -> str: ...
def repercent_broken_unicode(path: bytes) -> bytes: ...
@overload
def filepath_to_uri(path: None) -> None: ...
@overload
def filepath_to_uri(path: str) -> str: ...
def get_system_encoding() -> str: ...

DEFAULT_LOCALE_ENCODING: Any



from typing import Mapping, Iterable

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.#!/usr/bin/env python3
from ..util import _straight_line_vertices
import numpy as np

class NetworkxCoordinates:
    def __init__(self, graph=None, layout: str | dict | Iterable[np.float32] | None = None, **kwargs: Mapping): ...
    def __call__(self, adjacency_mat, directed: bool = False): ...
    @property
    def adj(self): ...



from networkx import Graph

from itertools import combinations

from ...classes.graph import Graph
from ...algorithms import chordal_graph_cliques, complete_to_chordal_graph, moral
from ...utils import not_implemented_for

__all__ = ["junction_tree"]

def junction_tree(G: Graph) -> Graph: ...



from .all import (
    union_all as union_all,
    compose_all as compose_all,
    disjoint_union_all as disjoint_union_all,
    intersection_all as intersection_all,
)
from .binary import (
    union as union,
    compose as compose,
    disjoint_union as disjoint_union,
    intersection as intersection,
    difference as difference,
    symmetric_difference as symmetric_difference,
    full_join as full_join,
)
from .product import (
    tensor_product as tensor_product,
    cartesian_product as cartesian_product,
    lexicographic_product as lexicographic_product,
    strong_product as strong_product,
    power as power,
    rooted_product as rooted_product,
)
from .unary import complement as complement, reverse as reverse



from typing import Any, ClassVar, Iterable, TypeVar
from ..utils.sparsefuncs import min_max_axis as min_max_axis
from collections import defaultdict as defaultdict
from numpy import ndarray
from numbers import Integral as Integral
from ..utils.multiclass import (
    unique_labels as unique_labels,
    type_of_target as type_of_target,
)
from pandas.core.frame import DataFrame
from ..base import BaseEstimator, TransformerMixin
from scipy.sparse import spmatrix, csr_matrix
from .._typing import ArrayLike, Int, MatrixLike, Float
from ..utils import column_or_1d as column_or_1d
from ..utils.validation import (
    check_array as check_array,
    check_is_fitted as check_is_fitted,
)

MultiLabelBinarizer_Self = TypeVar(
    "MultiLabelBinarizer_Self", bound="MultiLabelBinarizer"
)
LabelEncoder_Self = TypeVar("LabelEncoder_Self", bound="LabelEncoder")
LabelBinarizer_Self = TypeVar("LabelBinarizer_Self", bound="LabelBinarizer")

# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Mathieu Blondel <mathieu@mblondel.org>
#          Olivier Grisel <olivier.grisel@ensta.org>
#          Andreas Mueller <amueller@ais.uni-bonn.de>
#          Joel Nothman <joel.nothman@gmail.com>
#          Hamzeh Alsalhi <ha258@cornell.edu>
# License: BSD 3 clause

import itertools
import array
import warnings

import numpy as np
import scipy.sparse as sp


__all__ = [
    "label_binarize",
    "LabelBinarizer",
    "LabelEncoder",
    "MultiLabelBinarizer",
]


class LabelEncoder(TransformerMixin, BaseEstimator):
    classes_: ndarray = ...

    def fit(self: LabelEncoder_Self, y: ArrayLike) -> LabelEncoder_Self:
        ...

    def fit_transform(self, y: ArrayLike) -> ArrayLike:
        ...

    def transform(self, y: ArrayLike) -> ArrayLike:
        ...

    def inverse_transform(self, y: ArrayLike) -> ndarray:
        ...


class LabelBinarizer(TransformerMixin, BaseEstimator):
    sparse_input_: bool = ...
    y_type_: str = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, *, neg_label: Int = 0, pos_label: Int = 1, sparse_output: bool = False
    ) -> None:
        ...

    def fit(
        self: LabelBinarizer_Self, y: MatrixLike | ArrayLike
    ) -> LabelBinarizer_Self:
        ...

    def fit_transform(self, y: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...

    def transform(self, y: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...

    def inverse_transform(
        self, Y: MatrixLike, threshold: None | Float = None
    ) -> ndarray | spmatrix:
        ...


def label_binarize(
    y: ArrayLike | DataFrame,
    *,
    classes: ArrayLike,
    neg_label: Int = 0,
    pos_label: Int = 1,
    sparse_output: bool = False
) -> ndarray | spmatrix:
    ...


class MultiLabelBinarizer(TransformerMixin, BaseEstimator):
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, *, classes: None | ArrayLike = None, sparse_output: bool = False
    ) -> None:
        ...

    def fit(
        self: MultiLabelBinarizer_Self, y: Iterable[Iterable] | list[range]
    ) -> MultiLabelBinarizer_Self:
        ...

    def fit_transform(self, y: Iterable[Iterable]) -> ndarray | spmatrix:
        ...

    def transform(
        self, y: list[list[Any | Int]] | Iterable[Iterable]
    ) -> csr_matrix | ndarray:
        ...

    def inverse_transform(self, yt: MatrixLike) -> ndarray:
        ...



"""
This type stub file was generated by pyright.
"""

"""Pylab (matplotlib) support utilities."""
backends = ...
backend2gui = ...
def getfigs(*fig_nums):
    """Get a list of matplotlib figures by figure numbers.

    If no arguments are given, all available figures are returned.  If the
    argument list contains references to invalid figures, a warning is printed
    but the function continues pasting further figures.

    Parameters
    ----------
    figs : tuple
        A tuple of ints giving the figure numbers of the figures to return.
    """
    ...

def figsize(sizex, sizey): # -> None:
    """Set the default figure size to be [sizex, sizey].

    This is just an easy to remember, convenience wrapper that sets::

      matplotlib.rcParams['figure.figsize'] = [sizex, sizey]
    """
    ...

def print_figure(fig, fmt=..., bbox_inches=..., base64=..., **kwargs): # -> None:
    """Print a figure to an image, and return the resulting file data

    Returned data will be bytes unless ``fmt='svg'``,
    in which case it will be unicode.

    Any keyword args are passed to fig.canvas.print_figure,
    such as ``quality`` or ``bbox_inches``.

    If `base64` is True, return base64-encoded str instead of raw bytes
    for binary-encoded image formats

    .. versionadded:: 7.29
        base64 argument
    """
    ...

def retina_figure(fig, base64=..., **kwargs): # -> None:
    """format a figure as a pixel-doubled (retina) PNG

    If `base64` is True, return base64-encoded str instead of raw bytes
    for binary-encoded image formats

    .. versionadded:: 7.29
        base64 argument
    """
    ...

def mpl_runner(safe_execfile): # -> (fname: Unknown, *where: Unknown, **kw: Unknown) -> None:
    """Factory to return a matplotlib-enabled runner for %run.

    Parameters
    ----------
    safe_execfile : function
        This must be a function with the same interface as the
        :meth:`safe_execfile` method of IPython.

    Returns
    -------
    A function suitable for use as the ``runner`` argument of the %run magic
    function.
    """
    ...

def select_figure_formats(shell, formats, **kwargs): # -> None:
    """Select figure formats for the inline backend.

    Parameters
    ----------
    shell : InteractiveShell
        The main IPython instance.
    formats : str or set
        One or a set of figure formats to enable: 'png', 'retina', 'jpeg', 'svg', 'pdf'.
    **kwargs : any
        Extra keyword arguments to be passed to fig.canvas.print_figure.
    """
    ...

def find_gui_and_backend(gui=..., gui_select=...):
    """Given a gui string return the gui and mpl backend.

    Parameters
    ----------
    gui : str
        Can be one of ('tk','gtk','wx','qt','qt4','inline','agg').
    gui_select : str
        Can be one of ('tk','gtk','wx','qt','qt4','inline').
        This is any gui already selected by the shell.

    Returns
    -------
    A tuple of (gui, backend) where backend is one of ('TkAgg','GTKAgg',
    'WXAgg','Qt4Agg','module://matplotlib_inline.backend_inline','agg').
    """
    ...

def activate_matplotlib(backend): # -> None:
    """Activate the given backend and set interactive to True."""
    ...

def import_pylab(user_ns, import_all=...): # -> None:
    """Populate the namespace with pylab-related values.

    Imports matplotlib, pylab, numpy, and everything from pylab and numpy.

    Also imports a few names from IPython (figsize, display, getfigs)

    """
    ...

def configure_inline_support(shell, backend): # -> None:
    """
    .. deprecated:: 7.23

        use `matplotlib_inline.backend_inline.configure_inline_support()`

    Configure an IPython shell object for matplotlib use.

    Parameters
    ----------
    shell : InteractiveShell instance
    backend : matplotlib backend
    """
    ...




# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
from .shader_object import ShaderObject

class Expression(ShaderObject):
    def definition(self, names, version): ...

class TextExpression(Expression):
    def __init__(self, text): ...
    def __repr__(self): ...
    def expression(self, names=None): ...
    @property
    def text(self): ...
    @text.setter
    def text(self, t): ...
    def __eq__(self, a): ...
    def __hash__(self): ...

class FunctionCall(Expression):
    def __init__(self, function, args): ...
    def __repr__(self): ...
    @property
    def function(self): ...
    @property
    def dtype(self): ...
    def expression(self, names): ...



import warnings
from collections import OrderedDict

from .digraph import DiGraph
from .graph import Graph
from .multidigraph import MultiDiGraph
from .multigraph import MultiGraph

__all__ = []

class OrderedGraph(Graph):

    node_dict_factory = ...
    adjlist_outer_dict_factory = ...
    adjlist_inner_dict_factory = ...
    edge_attr_dict_factory = ...

    def __init__(self, incoming_graph_data=None, **attr): ...

class OrderedDiGraph(DiGraph):

    node_dict_factory = ...
    adjlist_outer_dict_factory = ...
    adjlist_inner_dict_factory = ...
    edge_attr_dict_factory = ...

    def __init__(self, incoming_graph_data=None, **attr): ...

class OrderedMultiGraph(MultiGraph):

    node_dict_factory = ...
    adjlist_outer_dict_factory = ...
    adjlist_inner_dict_factory = ...
    edge_key_dict_factory = ...
    edge_attr_dict_factory = ...

    def __init__(self, incoming_graph_data=None, **attr): ...

class OrderedMultiDiGraph(MultiDiGraph):

    node_dict_factory = ...
    adjlist_outer_dict_factory = ...
    adjlist_inner_dict_factory = ...
    edge_key_dict_factory = ...
    edge_attr_dict_factory = ...

    def __init__(self, incoming_graph_data=None, **attr): ...



"""
This type stub file was generated by pyright.
"""

""" Utilities for accessing the platform's clipboard.
"""
class ClipboardEmpty(ValueError):
    ...


def win32_clipboard_get():
    """ Get the current clipboard's text on Windows.

    Requires Mark Hammond's pywin32 extensions.
    """
    ...

def osx_clipboard_get() -> str:
    """ Get the clipboard's text on OS X.
    """
    ...

def tkinter_clipboard_get():
    """ Get the clipboard's text using Tkinter.

    This is the default on systems that are not Windows or OS X. It may
    interfere with other UI toolkits and should be replaced with an
    implementation that uses that toolkit.
    """
    ...

def wayland_clipboard_get():
    """Get the clipboard's text under Wayland using wl-paste command.

    This requires Wayland and wl-clipboard installed and running.
    """
    ...




from typing import Literal
from numpy.random import RandomState
from os.path import exists as exists, join as join
from ._base import RemoteFileMetadata as RemoteFileMetadata, load_descr as load_descr
from ..utils._bunch import Bunch
from gzip import GzipFile as GzipFile
from .._typing import Int
from ..utils import check_random_state as check_random_state, shuffle as shuffle_method
from . import get_data_home as get_data_home

import errno
import logging
import os

import numpy as np
import joblib


# The original data can be found at:
# https://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.data.gz
ARCHIVE = ...

# The original data can be found at:
# https://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.data_10_percent.gz
ARCHIVE_10_PERCENT = ...

logger = ...


def fetch_kddcup99(
    *,
    subset: None | Literal["SA", "SF", "http", "smtp"] = None,
    data_home: None | str = None,
    shuffle: bool = False,
    random_state: RandomState | None | Int = None,
    percent10: bool = True,
    download_if_missing: bool = True,
    return_X_y: bool = False,
    as_frame: bool = False,
) -> Bunch | tuple[Bunch, tuple]:
    ...



from typing import Mapping
from ...classes.graph import Graph

__all__ = ["ISMAGS"]

import itertools
from collections import Counter, defaultdict
from functools import reduce, wraps

def are_all_equal(iterable) -> bool: ...
def make_partitions(items, test): ...
def partition_to_color(partitions) -> Mapping: ...
def intersect(collection_of_sets) -> set: ...

class ISMAGS:
    def __init__(
        self,
        graph: Graph,
        subgraph: Graph,
        node_match=None,
        edge_match=None,
        cache=None,
    ): ...
    @property
    def _sgn_partitions(self): ...
    @property
    def _sge_partitions(self): ...
    @property
    def _gn_partitions(self): ...
    @property
    def _ge_partitions(self): ...
    @property
    def _sgn_colors(self): ...
    @property
    def _sge_colors(self): ...
    @property
    def _gn_colors(self): ...
    @property
    def _ge_colors(self): ...
    @property
    def _node_compatibility(self): ...
    @property
    def _edge_compatibility(self): ...
    @staticmethod
    def _node_match_maker(cmp): ...
    @staticmethod
    def _edge_match_maker(cmp): ...
    def find_isomorphisms(self, symmetry: bool = True): ...
    @staticmethod
    def _find_neighbor_color_count(graph, node, node_color, edge_color): ...
    def _get_lookahead_candidates(self): ...
    def largest_common_subgraph(self, symmetry: bool = True): ...
    def analyze_symmetry(self, graph: Graph, node_partitions, edge_colors): ...
    def is_isomorphic(self, symmetry=False) -> bool: ...
    def subgraph_is_isomorphic(self, symmetry=False) -> bool: ...
    def isomorphisms_iter(self, symmetry=True): ...
    def subgraph_isomorphisms_iter(self, symmetry=True): ...
    def _find_nodecolor_candidates(self): ...
    @staticmethod
    def _make_constraints(cosets): ...
    @staticmethod
    def _find_node_edge_color(graph, node_colors, edge_colors): ...
    @staticmethod
    def _get_permutations_by_length(items): ...
    @classmethod
    def _refine_node_partitions(
        cls, graph, node_partitions, edge_colors, branch=False
    ): ...
    def _edges_of_same_color(self, sgn1, sgn2): ...
    def _map_nodes(
        self, sgn, candidates, constraints, mapping=None, to_be_mapped=None
    ): ...
    def _largest_common_subgraph(self, candidates, constraints, to_be_mapped=None): ...
    @staticmethod
    def _remove_node(node, nodes, constraints): ...
    @staticmethod
    def _find_permutations(top_partitions, bottom_partitions): ...
    @staticmethod
    def _update_orbits(orbits, permutations): ...
    def _couple_nodes(
        self,
        top_partitions,
        bottom_partitions,
        pair_idx,
        t_node,
        b_node,
        graph,
        edge_colors,
    ): ...
    def _process_ordered_pair_partitions(
        self,
        graph,
        top_partitions,
        bottom_partitions,
        edge_colors,
        orbits=None,
        cosets=None,
    ): ...



from typing import Mapping
from itertools import chain, count

from ...classes.graph import Graph

__all__ = ["node_link_data", "node_link_graph"]

_attrs = ...

def node_link_data(
    G: Graph,
    attrs: Mapping | None = None,
    *,
    source: str = "source",
    target: str = "target",
    name: str = "id",
    key: str = "key",
    link: str = "links",
) -> Mapping: ...
def node_link_graph(
    data: Mapping,
    directed: bool = False,
    multigraph: bool = True,
    attrs: Mapping | None = None,
    *,
    source: str = "source",
    target: str = "target",
    name: str = "id",
    key: str = "key",
    link: str = "links",
): ...



"""
This type stub file was generated by pyright.
"""

"""
Context managers for adding things to sys.path temporarily.

Authors:

* Brian Granger
"""
class appended_to_syspath:
    """
    Deprecated since IPython 8.1, no replacements.

    A context for appending a directory to sys.path for a second."""
    def __init__(self, dir) -> None:
        ...
    
    def __enter__(self): # -> None:
        ...
    
    def __exit__(self, type, value, traceback):
        ...
    


class prepended_to_syspath:
    """A context for prepending a directory to sys.path for a second."""
    def __init__(self, dir) -> None:
        ...
    
    def __enter__(self): # -> None:
        ...
    
    def __exit__(self, type, value, traceback):
        ...
    





# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from ...util import transforms
from .perspective import Base3DRotationCamera

class TurntableCamera(Base3DRotationCamera):

    _state_props = ...

    def __init__(
        self,
        fov: float = 45.0,
        elevation: float = 30.0,
        azimuth: float = 30.0,
        roll: float = 0.0,
        distance: None | float = None,
        translate_speed: float = 1.0,
        **kwargs,
    ): ...
    @property
    def elevation(self): ...
    @elevation.setter
    def elevation(self, elev): ...
    @property
    def azimuth(self): ...
    @azimuth.setter
    def azimuth(self, azim): ...
    @property
    def roll(self): ...
    @roll.setter
    def roll(self, roll): ...
    def orbit(self, azim: float, elev: float): ...
    def _update_rotation(self, event): ...
    def _get_rotation_tr(self): ...
    def _dist_to_trans(self, dist): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

# Class adapted from:
# http://stackoverflow.com/questions/3603502/

class Frozen(object):
    __isfrozen: bool = ...

    def __setattr__(self, key, value): ...
    def freeze(self): ...
    def unfreeze(self): ...



from numpy.typing import ArrayLike
from ...classes.graph import Graph
from ...utils import not_implemented_for, pairwise

__all__ = ["is_semiconnected"]

def is_semiconnected(G: Graph, topo_order: ArrayLike | tuple | None = None) -> bool: ...



from collections import deque

from ...classes.graph import Graph

from .edmondskarp import edmonds_karp_core
from .utils import CurrentEdge, build_residual_network

__all__ = ["shortest_augmenting_path"]

def shortest_augmenting_path_impl(
    G: Graph, s, t, capacity, residual, two_phase, cutoff
): ...
def shortest_augmenting_path(
    G: Graph,
    s,
    t,
    capacity: str = "capacity",
    residual=None,
    value_only: bool = False,
    two_phase: bool = False,
    cutoff=None,
): ...



from typing import ClassVar, Iterable, Literal, Sequence, TypeVar
from ..base import BaseEstimator
from ..exceptions import NotFittedError as NotFittedError
from ..model_selection._split import BaseShuffleSplit
from ..preprocessing import LabelEncoder as LabelEncoder
from ..linear_model._logistic import LogisticRegression
from copy import deepcopy as deepcopy
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    column_or_1d as column_or_1d,
)
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from ..utils._param_validation import HasMethods as HasMethods, StrOptions as StrOptions
from numbers import Integral as Integral
from ..linear_model._ridge import RidgeCV
from ..base import (
    clone as clone,
    ClassifierMixin,
    RegressorMixin,
    TransformerMixin,
    is_classifier as is_classifier,
    is_regressor as is_regressor,
)
from ..model_selection import (
    cross_val_predict as cross_val_predict,
    check_cv as check_cv,
)
from ..utils import Bunch
from ..pipeline import Pipeline
from ._base import _BaseHeterogeneousEnsemble
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
    type_of_target as type_of_target,
)
from ..utils.metaestimators import available_if as available_if
from .._typing import MatrixLike, ArrayLike, Int
from ..model_selection import BaseCrossValidator

_BaseStacking_Self = TypeVar("_BaseStacking_Self", bound="_BaseStacking")
StackingRegressor_Self = TypeVar("StackingRegressor_Self", bound="StackingRegressor")
StackingClassifier_Self = TypeVar("StackingClassifier_Self", bound="StackingClassifier")


import numpy as np
import scipy.sparse as sparse


class _BaseStacking(TransformerMixin, _BaseHeterogeneousEnsemble, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        estimators,
        final_estimator=None,
        *,
        cv=None,
        stack_method: str = "auto",
        n_jobs=None,
        verbose: int = 0,
        passthrough: bool = False,
    ) -> None:
        ...

    def fit(
        self: _BaseStacking_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> _BaseStacking_Self | StackingClassifier:
        ...

    def n_features_in_(self):
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def predict(self, X: MatrixLike | ArrayLike, **predict_params) -> ndarray:
        ...


class StackingClassifier(ClassifierMixin, _BaseStacking):
    stack_method_: list[str] = ...
    final_estimator_: BaseEstimator = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    named_estimators_: Bunch = ...
    estimators_: list[BaseEstimator] = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimators: Sequence[tuple[str, BaseEstimator]],
        final_estimator: None | BaseEstimator | LogisticRegression = None,
        *,
        cv: int | BaseCrossValidator | Iterable | None | str | BaseShuffleSplit = None,
        stack_method: Literal[
            "auto", "predict_proba", "decision_function", "predict", "auto"
        ] = "auto",
        n_jobs: None | Int = None,
        passthrough: bool = False,
        verbose: Int = 0,
    ) -> None:
        ...

    def fit(
        self: StackingClassifier_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> StackingClassifier_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike, **predict_params) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray | list[ndarray]:
        ...

    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class StackingRegressor(RegressorMixin, _BaseStacking):
    stack_method_: list[str] = ...
    final_estimator_: BaseEstimator = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    named_estimators_: Bunch = ...
    estimators_: list[BaseEstimator] = ...

    def __init__(
        self,
        estimators: Sequence[tuple[str, BaseEstimator]] | list[tuple[str, Pipeline]],
        final_estimator: None | BaseEstimator | RidgeCV = None,
        *,
        cv: int | BaseCrossValidator | Iterable | None | str | BaseShuffleSplit = None,
        n_jobs: None | Int = None,
        passthrough: bool = False,
        verbose: Int = 0,
    ) -> None:
        ...

    def fit(
        self: StackingRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> StackingRegressor_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



from ..scene.node import Node

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
import re
import weakref

from .. import visuals
from .node import Node
from ..visuals.filters import Alpha, PickingFilter
from typing import ClassVar, Type, TypeVar

_T = ...

class VisualNode(Node):
    _next_id: int = ...
    _visual_ids = ...

    def __init__(self, parent: Node | None = None, name: str | None = None): ...
    def _update_opacity(self): ...
    def _set_clipper(self, node, clipper): ...
    @property
    def picking(self): ...
    @picking.setter
    def picking(self, p): ...
    def _update_trsys(self, event): ...
    @property
    def interactive(self): ...
    @interactive.setter
    def interactive(self, i): ...
    def draw(self): ...

def create_visual_node(subclass: _T) -> _T: ...
def generate_docstring(subclass, clsname): ...

# This is _not_ automated to help with auto-completion of IDEs,
# python REPL and IPython.
# Explicitly initializing these members allow IDEs to lookup
# and provide auto-completion. One problem is the fact that
# Docstrings are _not_ looked up correctly by IDEs, since they
# are attached programatically in the create_visual_node call.
# However, help(vispy.scene.FooVisual) still works

Arrow = visuals.ArrowVisual
Axis = visuals.AxisVisual
Box = visuals.BoxVisual
ColorBar = visuals.ColorBarVisual
Compound = visuals.CompoundVisual
Cube = visuals.CubeVisual
Ellipse = visuals.EllipseVisual
Graph = visuals.GraphVisual
GridLines = visuals.GridLinesVisual
GridMesh = visuals.GridMeshVisual
Histogram = visuals.HistogramVisual
Image = visuals.ImageVisual
ComplexImage = visuals.ComplexImageVisual
InfiniteLine = visuals.InfiniteLineVisual
Isocurve = visuals.IsocurveVisual
Isoline = visuals.IsolineVisual
Isosurface = visuals.IsosurfaceVisual
Line = visuals.LineVisual
LinearRegion = visuals.LinearRegionVisual
LinePlot = visuals.LinePlotVisual
Markers = visuals.MarkersVisual
Mesh = visuals.MeshVisual
MeshNormals = visuals.MeshNormalsVisual
Plane = visuals.PlaneVisual
Polygon = visuals.PolygonVisual 
Rectangle = visuals.RectangleVisual
RegularPolygon = visuals.RegularPolygonVisual
ScrollingLines = visuals.ScrollingLinesVisual
Spectrogram = visuals.SpectrogramVisual
Sphere = visuals.SphereVisual
SurfacePlot = visuals.SurfacePlotVisual
Text = visuals.TextVisual
Tube = visuals.TubeVisual
# Visual = create_visual_node(visuals.Visual)  # Should not be created
Volume = visuals.VolumeVisual
Windbarb = visuals.WindbarbVisual
XYZAxis = visuals.XYZAxisVisual



from typing import Literal
from numpy.random import RandomState
from ._base import RemoteFileMetadata as RemoteFileMetadata, load_descr as load_descr
from numpy import ndarray
from .. import preprocessing as preprocessing
from ..utils._bunch import Bunch
from .._typing import ArrayLike, Int
from ..utils import check_random_state as check_random_state
from ..feature_extraction.text import CountVectorizer as CountVectorizer
from . import get_data_home as get_data_home, load_files as load_files

# Copyright (c) 2011 Olivier Grisel <olivier.grisel@ensta.org>
# License: BSD 3 clause

import os
import logging
import tarfile
import pickle
import shutil
import re
import codecs

import numpy as np
import scipy.sparse as sp
import joblib

logger = ...

# The original data can be found at:
# https://people.csail.mit.edu/jrennie/20Newsgroups/20news-bydate.tar.gz
ARCHIVE = ...

CACHE_NAME: str = ...
TRAIN_FOLDER: str = ...
TEST_FOLDER: str = ...


def strip_newsgroup_header(text: str) -> str:
    ...


_QUOTE_RE = ...


def strip_newsgroup_quoting(text: str) -> str:
    ...


def strip_newsgroup_footer(text: str) -> str:
    ...


def fetch_20newsgroups(
    *,
    data_home: None | str = None,
    subset: Literal["train", "test", "all", "train"] = "train",
    categories: None | ArrayLike = None,
    shuffle: bool = True,
    random_state: RandomState | None | Int = 42,
    remove: tuple = ...,
    download_if_missing: bool = True,
    return_X_y: bool = False,
) -> Bunch | tuple[list[str], ndarray] | tuple[Bunch, tuple]:
    ...


def fetch_20newsgroups_vectorized(
    *,
    subset: Literal["train", "test", "all", "train"] = "train",
    remove: tuple = ...,
    data_home: None | str = None,
    download_if_missing: bool = True,
    return_X_y: bool = False,
    normalize: bool = True,
    as_frame: bool = False,
) -> tuple[Bunch, tuple]:
    ...



from collections.abc import Iterable
from numpy.typing import ArrayLike, NDArray
from typing import Any, Mapping, Literal
import inspect
import functools
import sys
import warnings
from collections.abc import Iterable

import numpy as np
import scipy
from numpy.lib import NumpyVersion

from ._warnings import all_warnings, warn

__all__ = [
    "deprecated",
    "get_bound_method_class",
    "all_warnings",
    "safe_as_int",
    "check_shape_equality",
    "check_nD",
    "warn",
    "reshape_nd",
    "identity",
    "slice_at_axis",
]

class skimage_deprecation(Warning):
    pass

def _get_stack_rank(func): ...
def _is_wrapped(func): ...
def _get_stack_length(func): ...

class _DecoratorBaseClass:

    _stack_length: dict = ...

    def get_stack_length(self, func): ...

class change_default_value(_DecoratorBaseClass):
    def __init__(
        self,
        arg_name: str,
        *,
        new_value: Any,
        changed_version: str,
        warning_msg: str | None = None
    ): ...
    def __call__(self, func): ...

class remove_arg(_DecoratorBaseClass):
    def __init__(
        self, arg_name: str, *, changed_version: str, help_msg: str | None = None
    ): ...
    def __call__(self, func): ...

def docstring_add_deprecated(
    func, kwarg_mapping: Mapping, deprecated_version: str
) -> str: ...

class deprecate_kwarg(_DecoratorBaseClass):
    def __init__(
        self,
        kwarg_mapping: Mapping,
        deprecated_version: str,
        warning_msg: str | None = None,
        removed_version: str | None = None,
    ): ...
    def __call__(self, func): ...

class deprecate_multichannel_kwarg(deprecate_kwarg):
    def __init__(self, removed_version: str = "1.0", multichannel_position=None): ...
    def __call__(self, func): ...

class channel_as_last_axis:
    def __init__(
        self,
        channel_arg_positions: tuple[int, ...] = ...,
        channel_kwarg_names: tuple[str, ...] = ...,
        multichannel_output: bool = True,
    ): ...
    def __call__(self, func): ...

class deprecated:
    def __init__(
        self,
        alt_func: str | None = None,
        behavior: Literal["warn", "raise"] = "warn",
        removed_version: str | None = None,
    ): ...
    def __call__(self, func): ...

def get_bound_method_class(m): ...
def safe_as_int(val, atol: float = 1e-3): ...
def check_shape_equality(im1, im2): ...
def slice_at_axis(sl, axis: int): ...
def reshape_nd(arr, ndim: int, dim: int): ...
def check_nD(array: ArrayLike, ndim: int | Iterable[int], arg_name: str = "image"): ...
def convert_to_float(image: NDArray, preserve_range: bool) -> NDArray: ...
def _validate_interpolation_order(image_dtype, order): ...
def _to_np_mode(mode): ...
def _to_ndimage_mode(mode): ...
def _fix_ndimage_mode(mode): ...

new_float_type: dict = ...

def _supported_float_type(input_dtype, allow_complex=False): ...
def identity(image, *args, **kwargs): ...



from collections import defaultdict

from ..classes.graph import Graph
from ..utils import not_implemented_for, pairwise

__all__ = [
    "cycle_basis",
    "simple_cycles",
    "recursive_simple_cycles",
    "find_cycle",
    "minimum_cycle_basis",
]

def cycle_basis(G: Graph, root=None): ...
def simple_cycles(G: Graph): ...
def recursive_simple_cycles(G: Graph): ...
def find_cycle(G: Graph, source=None, orientation=None): ...
def minimum_cycle_basis(G: Graph, weight: str | None = None): ...



from pandas.core.frame import DataFrame
from pandas.core.indexes.base import Index
from pandas.core.series import Series
from pandas._typing import FilePathOrBuffer as FilePathOrBuffer, FrameOrSeries as FrameOrSeries
from typing import Any, List, Optional, Union

lzma = ...
N: int = ...
K: int = ...

def set_testing_mode() -> None: ...
def reset_testing_mode() -> None: ...
def reset_display_options() -> None: ...
def round_trip_pickle(obj, path: Optional[FilePathOrBuffer]=...) -> FrameOrSeries: ...
def round_trip_pathlib(writer, reader, path: Optional[str]=...) : ...
def round_trip_localpath(writer, reader, path: Optional[str]=...) : ...
def decompress_file(path, compression) -> None: ...
def write_to_compressed(compression, path, data, dest: str = ...) -> None: ...
def assert_almost_equal(left, right, check_dtype: Union[bool, str]=..., check_less_precise: Union[bool, int]=..., **kwargs) : ...
def assert_dict_equal(left, right, compare_keys: bool=...) : ...
def randbool(size=..., p: float=...) : ...

RANDS_CHARS = ...
RANDU_CHARS = ...

def rands_array(nchars, size, dtype: str = ...): ...
def randu_array(nchars, size, dtype: str = ...): ...
def rands(nchars): ...
def randu(nchars): ...
def close(fignum = ...) -> None: ...
def ensure_clean(filename = ..., return_filelike: bool = ...) -> None: ...
def ensure_clean_dir() -> None: ...
def ensure_safe_environment_variables() -> None: ...
def equalContents(arr1, arr2) -> bool: ...
def assert_index_equal(left: Index[Any], right: Index[Any]) -> None: ...
def assert_class_equal(left, right, exact: Union[bool, str]=..., obj=...) : ...
def assert_attr_equal(attr, left, right, obj: str = ...): ...
def assert_is_valid_plot_return_object(objs) -> None: ...
def isiterable(obj) -> bool: ...
def assert_is_sorted(seq) -> None: ...
def assert_categorical_equal(left, right, check_dtype: bool = ..., check_category_order: bool = ..., obj: str = ...) -> None: ...
def assert_interval_array_equal(left, right, exact: str = ..., obj: str = ...) -> None: ...
def assert_period_array_equal(left, right, obj: str = ...) -> None: ...
def assert_datetime_array_equal(left, right, obj: str = ...) -> None: ...
def assert_timedelta_array_equal(left, right, obj: str = ...) -> None: ...
def raise_assert_detail(obj, message, left, right, diff = ...) -> None: ...
def assert_numpy_array_equal(left, right, strict_nan: bool = ..., check_dtype: bool = ..., err_msg = ..., check_same = ..., obj: str = ...): ...
def assert_extension_array_equal(
    left, right, check_dtype: bool = ..., check_less_precise: bool = ..., check_exact: bool = ...,
) -> None: ...
def assert_series_equal(left: Series, right: Series,
    check_dtype: bool = ...,
    check_index_type: bool|str = ...,
    check_series_type: bool = ...,
    check_less_precise: bool|int = ...,
    check_names : bool = ...,
    check_exact: bool = ...,
    check_datetimelike_compat: bool = ...,
    check_categorical: bool = ...,
    check_category_order: bool = ...,
    check_freq: bool = ...,
    check_flags: bool = ...,
    rtol: float = ...,
    atol: float = ...,
    obj: str = ...,
    *,
    check_index: bool = ...) -> None: ...
def assert_frame_equal(left: DataFrame, right: DataFrame, check_like: Optional[bool] = ...) -> None: ...
def assert_equal(left, right, **kwargs) -> None: ...
def box_expected(expected, box_cls, transpose: bool = ...): ...
def to_array(obj): ...
def assert_sp_array_equal(left, right, check_dtype: bool = ..., check_kind: bool = ..., check_fill_value: bool = ..., consolidate_block_indices: bool = ...) -> None: ...
def assert_contains_all(iterable, dic) -> None: ...
def assert_copy(iter1, iter2, **eql_kwargs) -> None: ...
def getCols(k): ...
def makeStringIndex(k: int = ..., name = ...): ...
def makeUnicodeIndex(k: int = ..., name = ...): ...
def makeCategoricalIndex(k: int = ..., n: int = ..., name = ..., **kwargs): ...
def makeIntervalIndex(k: int = ..., name = ..., **kwargs): ...
def makeBoolIndex(k: int = ..., name = ...): ...
def makeIntIndex(k: int = ..., name = ...): ...
def makeUIntIndex(k: int = ..., name = ...): ...
def makeRangeIndex(k: int = ..., name = ..., **kwargs): ...
def makeFloatIndex(k: int = ..., name = ...): ...
def makeDateIndex(k: int = ..., freq: str = ..., name = ..., **kwargs): ...
def makeTimedeltaIndex(k: int = ..., freq: str = ..., name = ..., **kwargs): ...
def makePeriodIndex(k: int = ..., name = ..., **kwargs): ...
def makeMultiIndex(k: int = ..., names = ..., **kwargs): ...
def all_index_generator(k: int = ...) -> None: ...
def index_subclass_makers_generator() -> None: ...
def all_timeseries_index_generator(k: int = ...) -> None: ...
def makeFloatSeries(name = ...): ...
def makeStringSeries(name = ...): ...
def makeObjectSeries(name = ...): ...
def getSeriesData(): ...
def makeTimeSeries(nper = ..., freq: str = ..., name = ...): ...
def makePeriodSeries(nper = ..., name = ...): ...
def getTimeSeriesData(nper = ..., freq: str = ...): ...
def getPeriodData(nper = ...): ...
def makeTimeDataFrame(nper = ..., freq: str = ...): ...
def makeDataFrame(): ...
def getMixedTypeDict(): ...
def makeMixedDataFrame(): ...
def makePeriodFrame(nper = ...): ...
def makeCustomIndex(nentries, nlevels, prefix: str = ..., names: bool = ..., ndupe_l = ..., idx_type = ...): ...
def makeCustomDataframe(nrows, ncols, c_idx_names: bool = ..., r_idx_names: bool = ..., c_idx_nlevels: int = ..., r_idx_nlevels: int = ..., data_gen_f = ..., c_ndupe_l = ..., r_ndupe_l = ..., dtype = ..., c_idx_type = ..., r_idx_type = ...): ...
def makeMissingCustomDataframe(nrows, ncols, density: float = ..., random_state = ..., c_idx_names: bool = ..., r_idx_names: bool = ..., c_idx_nlevels: int = ..., r_idx_nlevels: int = ..., data_gen_f = ..., c_ndupe_l = ..., r_ndupe_l = ..., dtype = ..., c_idx_type = ..., r_idx_type = ...): ...
def makeMissingDataframe(density: float = ..., random_state = ...): ...
def optional_args(decorator): ...
def can_connect(url, error_classes = ...): ...
def network(t, url: str = ..., raise_on_error = ..., check_before_test: bool = ..., error_classes = ..., skip_errnos = ..., _skip_on_messages = ...): ...
with_connectivity_check = network

def assert_produces_warning(expected_warning = ..., filter_level: str = ..., clear = ..., check_stacklevel: bool = ..., raise_on_extra_warnings: bool = ...) -> None: ...

def with_csv_dialect(name, **kwargs) -> None: ...
def use_numexpr(use, min_elements = ...) -> None: ...
def test_parallel(num_threads: int = ..., kwargs_list = ...): ...

def set_timezone(tz: str) : ...
def convert_rows_list_to_csv_str(rows_list: List[str]) : ...



import ctypes
from .gl2 import _lib, _get_gl_func

# void = glActiveTexture(GLenum texture)
def glActiveTexture(texture): ...

# void = glAttachShader(GLuint program, GLuint shader)
def glAttachShader(program, shader): ...

# void = glBindAttribLocation(GLuint program, GLuint index, GLchar* name)
def glBindAttribLocation(program, index, name): ...

# void = glBindBuffer(GLenum target, GLuint buffer)
def glBindBuffer(target, buffer): ...

# void = glBindFramebuffer(GLenum target, GLuint framebuffer)
def glBindFramebuffer(target, framebuffer): ...

# void = glBindRenderbuffer(GLenum target, GLuint renderbuffer)
def glBindRenderbuffer(target, renderbuffer): ...

# void = glBindTexture(GLenum target, GLuint texture)
def glBindTexture(target, texture): ...

# void = glBlendColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
def glBlendColor(red, green, blue, alpha): ...

# void = glBlendEquation(GLenum mode)
def glBlendEquation(mode): ...

# void = glBlendEquationSeparate(GLenum modeRGB, GLenum modeAlpha)
def glBlendEquationSeparate(modeRGB, modeAlpha): ...

# void = glBlendFunc(GLenum sfactor, GLenum dfactor)
def glBlendFunc(sfactor, dfactor): ...

# void = glBlendFuncSeparate(GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha, GLenum dstAlpha)
def glBlendFuncSeparate(srcRGB, dstRGB, srcAlpha, dstAlpha): ...

# void = glBufferData(GLenum target, GLsizeiptr size, GLvoid* data, GLenum usage)
def glBufferData(target, data, usage): ...

# void = glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, GLvoid* data)
def glBufferSubData(target, offset, data): ...

# GLenum = glCheckFramebufferStatus(GLenum target)
def glCheckFramebufferStatus(target): ...

# void = glClear(GLbitfield mask)
def glClear(mask): ...

# void = glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
def glClearColor(red, green, blue, alpha): ...

# void = glClearDepthf(GLclampf depth)
def glClearDepth(depth): ...

# void = glClearStencil(GLint s)
def glClearStencil(s): ...

# void = glColorMask(GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha)
def glColorMask(red, green, blue, alpha): ...

# void = glCompileShader(GLuint shader)
def glCompileShader(shader): ...

# void = glCompressedTexImage2D(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, GLvoid* data)
def glCompressedTexImage2D(target, level, internalformat, width, height, border, data): ...

# void = glCompressedTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, GLvoid* data)
def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, data): ...

# void = glCopyTexImage2D(GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border)
def glCopyTexImage2D(target, level, internalformat, x, y, width, height, border): ...

# void = glCopyTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
def glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height): ...

# GLuint = glCreateProgram()
def glCreateProgram(): ...

# GLuint = glCreateShader(GLenum type)
def glCreateShader(type): ...

# void = glCullFace(GLenum mode)
def glCullFace(mode): ...

# void = glDeleteBuffers(GLsizei n, GLuint* buffers)
def glDeleteBuffer(buffer): ...

# void = glDeleteFramebuffers(GLsizei n, GLuint* framebuffers)
def glDeleteFramebuffer(framebuffer): ...

# void = glDeleteProgram(GLuint program)
def glDeleteProgram(program): ...

# void = glDeleteRenderbuffers(GLsizei n, GLuint* renderbuffers)
def glDeleteRenderbuffer(renderbuffer): ...

# void = glDeleteShader(GLuint shader)
def glDeleteShader(shader): ...

# void = glDeleteTextures(GLsizei n, GLuint* textures)
def glDeleteTexture(texture): ...

# void = glDepthFunc(GLenum func)
def glDepthFunc(func): ...

# void = glDepthMask(GLboolean flag)
def glDepthMask(flag): ...

# void = glDepthRangef(GLclampf zNear, GLclampf zFar)
def glDepthRange(zNear, zFar): ...

# void = glDetachShader(GLuint program, GLuint shader)
def glDetachShader(program, shader): ...

# void = glDisable(GLenum cap)
def glDisable(cap): ...

# void = glDisableVertexAttribArray(GLuint index)
def glDisableVertexAttribArray(index): ...

# void = glDrawArrays(GLenum mode, GLint first, GLsizei count)
def glDrawArrays(mode, first, count): ...

# void = glDrawElements(GLenum mode, GLsizei count, GLenum type, GLvoid* indices)
def glDrawElements(mode, count, type, offset): ...

# void = glEnable(GLenum cap)
def glEnable(cap): ...

# void = glEnableVertexAttribArray(GLuint index)
def glEnableVertexAttribArray(index): ...

# void = glFinish()
def glFinish(): ...

# void = glFlush()
def glFlush(): ...

# void = glFramebufferRenderbuffer(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer)
def glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer): ...

# void = glFramebufferTexture2D(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
def glFramebufferTexture2D(target, attachment, textarget, texture, level): ...

# void = glFrontFace(GLenum mode)
def glFrontFace(mode): ...

# void = glGenBuffers(GLsizei n, GLuint* buffers)
def glCreateBuffer(): ...

# void = glGenFramebuffers(GLsizei n, GLuint* framebuffers)
def glCreateFramebuffer(): ...

# void = glGenRenderbuffers(GLsizei n, GLuint* renderbuffers)
def glCreateRenderbuffer(): ...

# void = glGenTextures(GLsizei n, GLuint* textures)
def glCreateTexture(): ...

# void = glGenerateMipmap(GLenum target)
def glGenerateMipmap(target): ...

# void = glGetActiveAttrib(GLuint program, GLuint index, GLsizei bufsize, GLsizei* length, GLint* size, GLenum* type, GLchar* name)
def glGetActiveAttrib(program, index): ...

# void = glGetActiveUniform(GLuint program, GLuint index, GLsizei bufsize, GLsizei* length, GLint* size, GLenum* type, GLchar* name)
def glGetActiveUniform(program, index): ...

# void = glGetAttachedShaders(GLuint program, GLsizei maxcount, GLsizei* count, GLuint* shaders)
def glGetAttachedShaders(program): ...

# GLint = glGetAttribLocation(GLuint program, GLchar* name)
def glGetAttribLocation(program, name): ...

# void = glGetBooleanv(GLenum pname, GLboolean* params)
def _glGetBooleanv(pname): ...

# void = glGetBufferParameteriv(GLenum target, GLenum pname, GLint* params)
def glGetBufferParameter(target, pname): ...

# GLenum = glGetError()
def glGetError(): ...

# void = glGetFloatv(GLenum pname, GLfloat* params)
def _glGetFloatv(pname): ...

# void = glGetFramebufferAttachmentParameteriv(GLenum target, GLenum attachment, GLenum pname, GLint* params)
def glGetFramebufferAttachmentParameter(target, attachment, pname): ...

# void = glGetIntegerv(GLenum pname, GLint* params)
def _glGetIntegerv(pname): ...

# void = glGetProgramInfoLog(GLuint program, GLsizei bufsize, GLsizei* length, GLchar* infolog)
def glGetProgramInfoLog(program): ...

# void = glGetProgramiv(GLuint program, GLenum pname, GLint* params)
def glGetProgramParameter(program, pname): ...

# void = glGetRenderbufferParameteriv(GLenum target, GLenum pname, GLint* params)
def glGetRenderbufferParameter(target, pname): ...

# void = glGetShaderInfoLog(GLuint shader, GLsizei bufsize, GLsizei* length, GLchar* infolog)
def glGetShaderInfoLog(shader): ...

# void = glGetShaderPrecisionFormat(GLenum shadertype, GLenum precisiontype, GLint* range, GLint* precision)
def glGetShaderPrecisionFormat(shadertype, precisiontype): ...

# void = glGetShaderSource(GLuint shader, GLsizei bufsize, GLsizei* length, GLchar* source)
def glGetShaderSource(shader): ...

# void = glGetShaderiv(GLuint shader, GLenum pname, GLint* params)
def glGetShaderParameter(shader, pname): ...

# GLubyte* = glGetString(GLenum name)
def glGetParameter(pname): ...

# void = glGetTexParameterfv(GLenum target, GLenum pname, GLfloat* params)
def glGetTexParameter(target, pname): ...

# void = glGetUniformfv(GLuint program, GLint location, GLfloat* params)
def glGetUniform(program, location): ...

# GLint = glGetUniformLocation(GLuint program, GLchar* name)
def glGetUniformLocation(program, name): ...

# void = glGetVertexAttribfv(GLuint index, GLenum pname, GLfloat* params)
def glGetVertexAttrib(index, pname): ...

# void = glGetVertexAttribPointerv(GLuint index, GLenum pname, GLvoid** pointer)
def glGetVertexAttribOffset(index, pname): ...

# void = glHint(GLenum target, GLenum mode)
def glHint(target, mode): ...

# GLboolean = glIsBuffer(GLuint buffer)
def glIsBuffer(buffer): ...

# GLboolean = glIsEnabled(GLenum cap)
def glIsEnabled(cap): ...

# GLboolean = glIsFramebuffer(GLuint framebuffer)
def glIsFramebuffer(framebuffer): ...

# GLboolean = glIsProgram(GLuint program)
def glIsProgram(program): ...

# GLboolean = glIsRenderbuffer(GLuint renderbuffer)
def glIsRenderbuffer(renderbuffer): ...

# GLboolean = glIsShader(GLuint shader)
def glIsShader(shader): ...

# GLboolean = glIsTexture(GLuint texture)
def glIsTexture(texture): ...

# void = glLineWidth(GLfloat width)
def glLineWidth(width): ...

# void = glLinkProgram(GLuint program)
def glLinkProgram(program): ...

# void = glPixelStorei(GLenum pname, GLint param)
def glPixelStorei(pname, param): ...

# void = glPolygonOffset(GLfloat factor, GLfloat units)
def glPolygonOffset(factor, units): ...

# void = glReadPixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid* pixels)
def glReadPixels(x, y, width, height, format, type): ...

# void = glRenderbufferStorage(GLenum target, GLenum internalformat, GLsizei width, GLsizei height)
def glRenderbufferStorage(target, internalformat, width, height): ...

# void = glSampleCoverage(GLclampf value, GLboolean invert)
def glSampleCoverage(value, invert): ...

# void = glScissor(GLint x, GLint y, GLsizei width, GLsizei height)
def glScissor(x, y, width, height): ...

# void = glShaderSource(GLuint shader, GLsizei count, GLchar** string, GLint* length)
def glShaderSource(shader, source): ...

# void = glStencilFunc(GLenum func, GLint ref, GLuint mask)
def glStencilFunc(func, ref, mask): ...

# void = glStencilFuncSeparate(GLenum face, GLenum func, GLint ref, GLuint mask)
def glStencilFuncSeparate(face, func, ref, mask): ...

# void = glStencilMask(GLuint mask)
def glStencilMask(mask): ...

# void = glStencilMaskSeparate(GLenum face, GLuint mask)
def glStencilMaskSeparate(face, mask): ...

# void = glStencilOp(GLenum fail, GLenum zfail, GLenum zpass)
def glStencilOp(fail, zfail, zpass): ...

# void = glStencilOpSeparate(GLenum face, GLenum fail, GLenum zfail, GLenum zpass)
def glStencilOpSeparate(face, fail, zfail, zpass): ...

# void = glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, GLvoid* pixels)
def glTexImage2D(target, level, internalformat, format, type, pixels): ...
def glTexParameterf(target, pname, param): ...
def glTexParameteri(target, pname, param): ...

# void = glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid* pixels)
def glTexSubImage2D(target, level, xoffset, yoffset, format, type, pixels): ...
def glUniform1f(location, v1): ...
def glUniform2f(location, v1, v2): ...
def glUniform3f(location, v1, v2, v3): ...
def glUniform4f(location, v1, v2, v3, v4): ...
def glUniform1i(location, v1): ...
def glUniform2i(location, v1, v2): ...
def glUniform3i(location, v1, v2, v3): ...
def glUniform4i(location, v1, v2, v3, v4): ...
def glUniform1fv(location, count, values): ...
def glUniform2fv(location, count, values): ...
def glUniform3fv(location, count, values): ...
def glUniform4fv(location, count, values): ...
def glUniform1iv(location, count, values): ...
def glUniform2iv(location, count, values): ...
def glUniform3iv(location, count, values): ...
def glUniform4iv(location, count, values): ...
def glUniformMatrix2fv(location, count, transpose, values): ...
def glUniformMatrix3fv(location, count, transpose, values): ...
def glUniformMatrix4fv(location, count, transpose, values): ...

# void = glUseProgram(GLuint program)
def glUseProgram(program): ...

# void = glValidateProgram(GLuint program)
def glValidateProgram(program): ...
def glVertexAttrib1f(index, v1): ...
def glVertexAttrib2f(index, v1, v2): ...
def glVertexAttrib3f(index, v1, v2, v3): ...
def glVertexAttrib4f(index, v1, v2, v3, v4): ...

# void = glVertexAttribPointer(GLuint indx, GLint size, GLenum type, GLboolean normalized, GLsizei stride, GLvoid* ptr)
def glVertexAttribPointer(indx, size, type, normalized, stride, offset): ...

# void = glViewport(GLint x, GLint y, GLsizei width, GLsizei height)
def glViewport(x, y, width, height): ...



"""
This type stub file was generated by pyright.
"""

import os

"""
Utilities for working with terminals.

Authors:

* Brian E. Granger
* Fernando Perez
* Alexander Belchenko (e-mail: bialix AT ukr.net)
"""
ignore_termtitle = ...
if os.name == 'posix':
    ...
else:
    ...
def toggle_set_term_title(val): # -> None:
    """Control whether set_term_title is active or not.

    set_term_title() allows writing to the console titlebar.  In embedded
    widgets this can cause problems, so this call can be used to toggle it on
    or off as needed.

    The default state of the module is for the function to be disabled.

    Parameters
    ----------
    val : bool
        If True, set_term_title() actually writes to the terminal (using the
        appropriate platform-specific module).  If False, it is a no-op.
    """
    ...

_xterm_term_title_saved = ...
if os.name == 'posix':
    ...
else:
    SetConsoleTitleW = ...
def set_term_title(title): # -> None:
    """Set terminal title using the necessary platform-dependent calls."""
    ...

def restore_term_title(): # -> None:
    """Restore, if possible, terminal title to the original state"""
    ...

def freeze_term_title(): # -> None:
    ...

def get_terminal_size(defaultx=..., defaulty=...):
    ...




from typing import Mapping
from numpy.typing import ArrayLike
from ...classes.graph import Graph

__all__ = [
    "bidirectional_shortest_path",
    "single_source_shortest_path",
    "single_source_shortest_path_length",
    "single_target_shortest_path",
    "single_target_shortest_path_length",
    "all_pairs_shortest_path",
    "all_pairs_shortest_path_length",
    "predecessor",
]

def single_source_shortest_path_length(G: Graph, source, cutoff=None) -> Mapping: ...
def single_target_shortest_path_length(G: Graph, target, cutoff=None): ...
def all_pairs_shortest_path_length(G: Graph, cutoff=None): ...
def bidirectional_shortest_path(G: Graph, source, target) -> ArrayLike: ...
def single_source_shortest_path(G: Graph, source, cutoff=None) -> Mapping: ...
def single_target_shortest_path(G: Graph, target, cutoff=None) -> Mapping: ...
def all_pairs_shortest_path(G: Graph, cutoff=None) -> Mapping: ...
def predecessor(
    G: Graph, source, target=None, cutoff=None, return_seen=None
) -> Mapping: ...



from typing import Sequence
from ._layoutgrid import LayoutGrid
from .figure import Figure, FigureBase
from .gridspec import GridSpecBase

class LayoutEngine:
    def __init__(self, **kwargs) -> None: ...
    def set(self, **kwargs)-> None: ...
    @property
    def colorbar_gridspec(self) -> bool: ...
    @property
    def adjust_compatible(self) -> bool: ...
    def get(self): ...
    def execute(self, fig: Figure): ...

class TightLayoutEngine(LayoutEngine):
    def __init__(
        self,
        *,
        pad: float = 1.08,
        h_pad: float = ...,
        w_pad: float = ...,
        rect: Sequence[float] = ...,
        **kwargs
    ) -> None: ...
    def execute(self, fig: Figure) -> None: ...
    def set(
        self,
        *,
        pad: float = 1.08,
        w_pad: float = ...,
        h_pad: float = ...,
        rect: Sequence[float] = ...
    ) -> None: ...

class ConstrainedLayoutEngine(LayoutEngine):
    def __init__(
        self,
        *,
        h_pad: float = ...,
        w_pad: float = ...,
        hspace: float = ...,
        wspace: float = ...,
        rect: Sequence[float] = ...,
        compress: bool = ...,
        **kwargs
    ) -> None: ...
    def execute(
        self, fig: Figure
    ) -> dict[str | FigureBase | GridSpecBase, bool | LayoutGrid]: ...
    def set(
        self,
        *,
        h_pad: float = ...,
        w_pad: float = ...,
        hspace: float = ...,
        wspace: float = ...,
        rect: tuple = ...
    ) -> None: ...



class BaseGLProxy(object):
    def __call__(self, funcname, returns, *args): ...
    def glActiveTexture(self, texture): ...
    def glAttachShader(self, program, shader): ...
    def glBindAttribLocation(self, program, index, name): ...
    def glBindBuffer(self, target, buffer): ...
    def glBindFramebuffer(self, target, framebuffer): ...
    def glBindRenderbuffer(self, target, renderbuffer): ...
    def glBindTexture(self, target, texture): ...
    def glBlendColor(self, red, green, blue, alpha): ...
    def glBlendEquation(self, mode): ...
    def glBlendEquationSeparate(self, modeRGB, modeAlpha): ...
    def glBlendFunc(self, sfactor, dfactor): ...
    def glBlendFuncSeparate(self, srcRGB, dstRGB, srcAlpha, dstAlpha): ...
    def glBufferData(self, target, data, usage): ...
    def glBufferSubData(self, target, offset, data): ...
    def glCheckFramebufferStatus(self, target): ...
    def glClear(self, mask): ...
    def glClearColor(self, red, green, blue, alpha): ...
    def glClearDepth(self, depth): ...
    def glClearStencil(self, s): ...
    def glColorMask(self, red, green, blue, alpha): ...
    def glCompileShader(self, shader): ...
    def glCompressedTexImage2D(self, target, level, internalformat, width, height, border, data): ...
    def glCompressedTexSubImage2D(self, target, level, xoffset, yoffset, width, height, format, data): ...
    def glCopyTexImage2D(self, target, level, internalformat, x, y, width, height, border): ...
    def glCopyTexSubImage2D(self, target, level, xoffset, yoffset, x, y, width, height): ...
    def glCreateProgram(
        self,
    ): ...
    def glCreateShader(self, type): ...
    def glCullFace(self, mode): ...
    def glDeleteBuffer(self, buffer): ...
    def glDeleteFramebuffer(self, framebuffer): ...
    def glDeleteProgram(self, program): ...
    def glDeleteRenderbuffer(self, renderbuffer): ...
    def glDeleteShader(self, shader): ...
    def glDeleteTexture(self, texture): ...
    def glDepthFunc(self, func): ...
    def glDepthMask(self, flag): ...
    def glDepthRange(self, zNear, zFar): ...
    def glDetachShader(self, program, shader): ...
    def glDisable(self, cap): ...
    def glDisableVertexAttribArray(self, index): ...
    def glDrawArrays(self, mode, first, count): ...
    def glDrawElements(self, mode, count, type, offset): ...
    def glEnable(self, cap): ...
    def glEnableVertexAttribArray(self, index): ...
    def glFinish(
        self,
    ): ...
    def glFlush(
        self,
    ): ...
    def glFramebufferRenderbuffer(self, target, attachment, renderbuffertarget, renderbuffer): ...
    def glFramebufferTexture2D(self, target, attachment, textarget, texture, level): ...
    def glFrontFace(self, mode): ...
    def glCreateBuffer(
        self,
    ): ...
    def glCreateFramebuffer(
        self,
    ): ...
    def glCreateRenderbuffer(
        self,
    ): ...
    def glCreateTexture(
        self,
    ): ...
    def glGenerateMipmap(self, target): ...
    def glGetActiveAttrib(self, program, index): ...
    def glGetActiveUniform(self, program, index): ...
    def glGetAttachedShaders(self, program): ...
    def glGetAttribLocation(self, program, name): ...
    def _glGetBooleanv(self, pname): ...
    def glGetBufferParameter(self, target, pname): ...
    def glGetError(
        self,
    ): ...
    def _glGetFloatv(self, pname): ...
    def glGetFramebufferAttachmentParameter(self, target, attachment, pname): ...
    def _glGetIntegerv(self, pname): ...
    def glGetProgramInfoLog(self, program): ...
    def glGetProgramParameter(self, program, pname): ...
    def glGetRenderbufferParameter(self, target, pname): ...
    def glGetShaderInfoLog(self, shader): ...
    def glGetShaderPrecisionFormat(self, shadertype, precisiontype): ...
    def glGetShaderSource(self, shader): ...
    def glGetShaderParameter(self, shader, pname): ...
    def glGetParameter(self, pname): ...
    def glGetTexParameter(self, target, pname): ...
    def glGetUniform(self, program, location): ...
    def glGetUniformLocation(self, program, name): ...
    def glGetVertexAttrib(self, index, pname): ...
    def glGetVertexAttribOffset(self, index, pname): ...
    def glHint(self, target, mode): ...
    def glIsBuffer(self, buffer): ...
    def glIsEnabled(self, cap): ...
    def glIsFramebuffer(self, framebuffer): ...
    def glIsProgram(self, program): ...
    def glIsRenderbuffer(self, renderbuffer): ...
    def glIsShader(self, shader): ...
    def glIsTexture(self, texture): ...
    def glLineWidth(self, width): ...
    def glLinkProgram(self, program): ...
    def glPixelStorei(self, pname, param): ...
    def glPolygonOffset(self, factor, units): ...
    def glReadPixels(self, x, y, width, height, format, type): ...
    def glRenderbufferStorage(self, target, internalformat, width, height): ...
    def glSampleCoverage(self, value, invert): ...
    def glScissor(self, x, y, width, height): ...
    def glShaderSource(self, shader, source): ...
    def glStencilFunc(self, func, ref, mask): ...
    def glStencilFuncSeparate(self, face, func, ref, mask): ...
    def glStencilMask(self, mask): ...
    def glStencilMaskSeparate(self, face, mask): ...
    def glStencilOp(self, fail, zfail, zpass): ...
    def glStencilOpSeparate(self, face, fail, zfail, zpass): ...
    def glTexImage2D(self, target, level, internalformat, format, type, pixels): ...
    def glTexParameterf(self, target, pname, param): ...
    def glTexParameteri(self, target, pname, param): ...
    def glTexSubImage2D(self, target, level, xoffset, yoffset, format, type, pixels): ...
    def glUniform1f(self, location, v1): ...
    def glUniform2f(self, location, v1, v2): ...
    def glUniform3f(self, location, v1, v2, v3): ...
    def glUniform4f(self, location, v1, v2, v3, v4): ...
    def glUniform1i(self, location, v1): ...
    def glUniform2i(self, location, v1, v2): ...
    def glUniform3i(self, location, v1, v2, v3): ...
    def glUniform4i(self, location, v1, v2, v3, v4): ...
    def glUniform1fv(self, location, count, values): ...
    def glUniform2fv(self, location, count, values): ...
    def glUniform3fv(self, location, count, values): ...
    def glUniform4fv(self, location, count, values): ...
    def glUniform1iv(self, location, count, values): ...
    def glUniform2iv(self, location, count, values): ...
    def glUniform3iv(self, location, count, values): ...
    def glUniform4iv(self, location, count, values): ...
    def glUniformMatrix2fv(self, location, count, transpose, values): ...
    def glUniformMatrix3fv(self, location, count, transpose, values): ...
    def glUniformMatrix4fv(self, location, count, transpose, values): ...
    def glUseProgram(self, program): ...
    def glValidateProgram(self, program): ...
    def glVertexAttrib1f(self, index, v1): ...
    def glVertexAttrib2f(self, index, v1, v2): ...
    def glVertexAttrib3f(self, index, v1, v2, v3): ...
    def glVertexAttrib4f(self, index, v1, v2, v3, v4): ...
    def glVertexAttribPointer(self, indx, size, type, normalized, stride, offset): ...
    def glViewport(self, x, y, width, height): ...



from .beamsearch import bfs_beam_edges as bfs_beam_edges
from .breadth_first_search import (
    bfs_edges as bfs_edges,
    bfs_tree as bfs_tree,
    bfs_predecessors as bfs_predecessors,
    bfs_successors as bfs_successors,
    descendants_at_distance as descendants_at_distance,
    bfs_layers as bfs_layers,
)
from .depth_first_search import (
    dfs_edges as dfs_edges,
    dfs_tree as dfs_tree,
    dfs_predecessors as dfs_predecessors,
    dfs_successors as dfs_successors,
    dfs_preorder_nodes as dfs_preorder_nodes,
    dfs_postorder_nodes as dfs_postorder_nodes,
    dfs_labeled_edges as dfs_labeled_edges,
)
from .edgedfs import edge_dfs as edge_dfs
from .edgebfs import edge_bfs as edge_bfs



from typing import Mapping
from itertools import combinations

from networkx import NetworkXError
from ...classes.graph import Graph

__all__ = ["node_redundancy"]

def node_redundancy(G: Graph, nodes=None) -> Mapping: ...



from ..gloo.buffer import DataBuffer
from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np
from os import path as op
from traceback import extract_stack, format_list
import weakref

from .globject import GLObject
from ..util import logger

# ------------------------------------------------------------ Buffer class ---
class Buffer(GLObject):
    def __init__(self, data: NDArray | None = None, nbytes: None | int = None): ...
    @property
    def nbytes(self): ...
    def set_subdata(self, data: NDArray, offset: int = 0, copy: bool = False): ...
    def set_data(self, data: NDArray, copy: bool = False): ...
    def resize_bytes(self, size: int): ...

# -------------------------------------------------------- DataBuffer class ---
class DataBuffer(Buffer):
    def __init__(self, data: NDArray | None = None): ...
    def _prepare_data(self, data): ...
    def set_subdata(self, data: NDArray, offset: int = 0, copy: bool = False, **kwargs): ...
    def set_data(self, data: NDArray, copy: bool = False, **kwargs): ...
    @property
    def dtype(self): ...
    @property
    def offset(self): ...
    @property
    def stride(self): ...
    @property
    def size(self): ...
    @property
    def itemsize(self): ...
    @property
    def glsl_type(self): ...
    def resize_bytes(self, size: int): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, data): ...
    def __repr__(self): ...

class DataBufferView(DataBuffer):

    # Note that this class is a bit evil: it is a subclass of GLObject,
    # Buffer and DataBuffer, but any of these __init__'s are not called ...

    def __init__(self, base: DataBuffer, key: str | int | slice): ...
    @property
    def glir(self): ...
    @property
    def id(self): ...
    @property
    def _last_dim(self): ...
    def set_subdata(self, data: NDArray, offset: int = 0, copy: bool = False, **kwargs): ...
    def set_data(self, data: NDArray, copy: bool = False, **kwargs): ...
    @property
    def offset(self): ...
    @property
    def base(self): ...
    def resize_bytes(self, size: int): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, data): ...
    def __repr__(self): ...

# ------------------------------------------------------ VertexBuffer class ---
class VertexBuffer(DataBuffer):

    _GLIR_TYPE: str = ...

    def _prepare_data(self, data, convert=False): ...

def _last_stack_str(): ...

# ------------------------------------------------------- IndexBuffer class ---
class IndexBuffer(DataBuffer):

    _GLIR_TYPE: str = ...

    def __init__(self, data: NDArray | None = None): ...
    def _prepare_data(self, data, convert=False): ...



from typing import Mapping
import math

from ...utils import not_implemented_for
from ...classes.graph import Graph

__all__ = ["katz_centrality", "katz_centrality_numpy"]

def katz_centrality(
    G: Graph,
    alpha: float = 0.1,
    beta=1.0,
    max_iter=1000,
    tol=1.0e-6,
    nstart: Mapping | None = None,
    normalized=True,
    weight=None,
) -> Mapping: ...
def katz_centrality_numpy(
    G: Graph,
    alpha: float = 0.1,
    beta=1.0,
    normalized: bool = True,
    weight: str | None = None,
) -> Mapping: ...
